###############################################################################
## Help System 2.2.4
## KobazHelp.tcl
## 
## AUTHOR: Kobaz Mi'Daire <krobaz@yahoo.com>
## 
## REQUIRES: Eggdrop 1.3.x AND tcl 8.1
##           KobazDatabaseAPI.tcl
##           KobazConfig.tcl
##           KobazTools.tcl
##
## PURPOSE:  An easy to manage help system for eggdrop
##
## COMMANDS: Arguments in "< >" are mandatory.
##           Arguments in "? ?" are optional.  
##           Arguments in "[ ]" are a toggle, and mandatory.
##           Arguments in "( )" must be used together
## 
## - Add help entry
## addhelp <command> <brief info> <flag available to>  
##
## - Add more in depth help <not complete>
## addhelp_extra <command> <info> <flag available to> 
###############################################################################

###############################################################################
# All Configuration is done via the bot now: the .kbconfig KobazHelp command
# If help commands do not work in a specific channel, remember to use
# .kbconfig KobazHelp to set enable to 1 for that channel
###############################################################################

####
# Load required libraries

source scripts/kobazscripts/KobazDep.tcl
kbrequiretcl 8.1

# end
####


namespace eval KobazHelp {
set a "KobazHelp"

namespace export addhelp
namespace export addhelp_extra

kbprovide $a
kbrequire DatabaseAPI
kbrequire Config
kbrequire Tools

namespace import -force ::KobazDB::*
namespace import -force ::KobazTools::*
namespace import -force ::KobazConfig::*

############################
# Settings - Do NOT Modify #
############################
proc add_configs {} {
variable a
add_config $a CREATE . . .
add_config $a "tog global" debug "0" {Enable debug? You probebly really don't need to turn this on}
add_config $a "tog" enable "1" {Enable help commands on this channel?}
}
#################
# Settings -End #
#################

if {![info exists firstload]} {
    add_configs
    dbCreate help KobazHelp.db {name cmd def extra} nosave
    if {[catch {load_config $a}]} {
        setup_globals $a
    }
    set firstload  "1"
}

############################################
############################################
## Modify contents below at your own risk ##
############################################
############################################

proc init {} {
    variable a

    bind pub - !help       KobazHelp::pub_help

    loadall_configs $a

    putlog "$a.tcl by Kobaz loaded."
    return
}

proc error {args} {
    putlog "HELP=:=ERROR $args"
}

proc debug {args} {
    variable a
    set debug [get_config $a debug]
    if {$debug == "1"} {
	putlog "HELP:DEBUG $args"
    }
}

proc addhelp {name cmd def} {
    variable data
    
    if {![info exists data($name)]} {set data($name) "1"}
    
    if {[dbBinarySearch help cmd $cmd] != "-1"} {
        # Help entry already exists
	return
    }

    set entry ""
    lappend entry $name
    lappend entry $cmd
    lappend entry $def
    lappend entry ""
    dbAppend help $entry
    dbSort help cmd

    return
}

proc addhelp_extra {name cmd extra} {
    set num [dbBinarySearch help cmd $cmd]

    if {$num == "-1"} {
	return -code error "addhelp_extra: cmd $name dosn't exist"
    }

    if {[dbField help extra $num] != ""} {
	return -code error "addhelp_extra: extra info for $name exists"
    }

    # Delete entry so we can readd it with the
    # extra field filled out
    set name [dbField help name $num]
    set cmd  [dbField help cmd  $num]
    set def  [dbField help def  $num]
    dbDelete help $num

    set entry ""
    lappend entry $name
    lappend entry $cmd
    lappend entry $def
    lappend entry $extra
    dbAppend help $entry
    dbSort help cmd

    return
}

proc pub_help {nick uhost handle channel arg} {
    set channel [string tolower $channel]
    variable a
    if {[get_config ${a}_$channel enable] == "0"} {return}
    variable data
    set type notice

    if {[dbNumRecs help] == "0"} {
	KobazTools::tellnick $type $nick "No Help Available."
	return
    }	

    if {$arg != ""} {
	set cmdnum [dbBinarySearch help cmd $arg]
	if {$cmdnum == "-1"} {
	    tellnick $type $nick "No extra help available for: ${arg}"
	    return
	}

	set cmd    [dbField help cmd  $cmdnum]
	set name   [dbField help name $cmdnum]
	set flag   [get_config ${name}_$channel $cmd]
	
	if {[check_access $nick $channel $handle $flag ""]} {
	    tellnick $type $nick "Help for: ${arg}"
	    tellnick $type $nick "[dbField help cmd $cmdnum]  - [dbField help extra $cmdnum]"
	} else {
	    return
	}
    }

    if {$arg == ""} {
	set first 0

	foreach name [array names data] {
	    if {$first != "1"} {
		tellnick $type $nick "The Kobaz Help System"
		tellnick $type $nick "* Indicates more help by typing !help <command>"
		tellnick $type $nick "Command  Info"
		set first 1
	    }
	    set helplist [dbMatchingRecords help name $name]
	    if {$helplist == ""} {continue}
	    set second 0
	    foreach entry $helplist {
		set cmd  [dbField help cmd $entry]
		set flag   [get_config ${name}_$channel $cmd]
		if {$flag == ""} {continue}
		if {[check_access $nick $channel $handle $flag ""]} {
		    if {$second != "1"} {
			tellnick $type $nick " -=- $name -=-"
			set second 1
		    }
		    set def  [dbField help def $entry]
		    if {[dbField help extra $entry] != ""} {set def "* $def"}
		    tellnick $type $nick "${cmd}  -${def}"
		}
	    }
	}
	return
    }
    return    
}

init
}
