###############################################################################
## Information Service System 2.2.4
## KobazLearn.tcl
##
## AUTHOR: Kobaz Mi'Daire <krobaz@yahoo.com>
## 
## REQUIRES: Eggdrop 1.3.x AND tcl 8.1
##           KobazDatabase.tcl
##           KobazDatabaseAPI.tcl
##           KobazSQL-Lib.tcl
##           KobazConfig.tcl
##           KobazTools.tcl
##           KobazHelp.tcl
##
## PURPOSE:  To provide an information lookup service
## 
## FEATURES: Ability to add entries.
##           Ability to remove entris.
##           Ability to search through entries.
##           Ability to "lock" an entry and prevent its deletion and/or 
##           modification.
##           Ability to "unlock" an entry and allow deletion and/or 
##           modification.
##           Ability to view when an entry was added, and by who.
##           Completely Customizeable via dcc.
## 
## COMMANDS: Arguments in "< >" are mandatory.
##           Arguments in "? ?" are optional.
## 
## !learn <flag> ?key? ?defination?
##         flags are: -a  -add
##                    -m  -modify
##                    -d  -delete
##                    -l  -lock
##                    -u  -unlock
##                    -h  -help
## ?? <key>
## ?? -i key
## ?? -t <nick> <key>
## !listkeys <search>
##
###############################################################################

###############################################################################
# All Configuration is done via the bot now: the .kbconfig KobazLearn command
# If learn commands do not work in a specific channel, remember to use
# .kbconfig KobazLearn to set enable to 1 for that channel
###############################################################################

############################################
############################################
## Modify contents below at your own risk ##
############################################
############################################


source scripts/kobazscripts/KobazDep.tcl
kbrequiretcl 8.1

namespace eval KobazLearn {
set a "KobazLearn"

####
# Load required libraries

kbrequire DatabaseAPI
kbrequire Config
kbrequire Tools
kbrequire Help

# end
####

namespace import -force ::KobazTools::*
namespace import -force ::KobazHelp::*
namespace import -force ::KobazConfig::*
namespace import -force ::KobazDBAPI::*

############################
# Settings - Do NOT Modify #
############################
proc add_configs {} {
variable a
add_config $a CREATE . . .
add_config $a "tog global" debug "0" {Enable debug? You probebly really don't need to turn this on}

add_config $a "tog" enable "0" {Enable learn commands on this channel?}
add_config $a "tog" usehelp "1" {Use public !help command?}
add_config $a "tog" nocolors "1"  {Remove colors of added entrys?}
add_config $a "tog" color_notify "1" {Notify people if colors arent allowed?}
add_config $a "num_no0" displaymax "20" {Maximux keys to display when doing !listkeys}
add_config $a "num_no0" maxkeylength "20" {Maximux size of a key}
add_config $a "tog" dropcase "1" {Convert all key names to lowercase? test and TEST will be the same key.}
add_config $a "text" datafile "KobazLearn.db" {Data file to use for this channel}
add_config $a flag ??                "-"  "Flags to access ??"
add_config $a flag !listkeys         "-"  "Flags to access !listkeys"
add_config $a flag "!learn -a"       "v"  "Flags to access !learn -a"
add_config $a flag "!learn -m"       "o"  "Flags to access !learn -m"
add_config $a flag "!learn -d"       "o"  "Flags to access !learn -d"
add_config $a flag "!learn -l"       "cm" "Flags to access !learn -l"
add_config $a flag "!learn -u"       "cm" "Flags to access !learn -u"
add_config $a flag "flagoveride"     "cm" "Flags to overide locked keys"
add_config $a text_star msg "No Access." {Message to give person if denied access to a command. If value is * then no message will be given.}
}
#################
# Settings -End #
#################

#################
# Help          #
#################
proc add_helps {} {
variable a
addhelp $a "??"              "See: ?? -h"
addhelp $a "!learn -a"       "Add entry"
addhelp $a "!learn -m"       "Modify entry"
addhelp $a "!learn -d"       "Delete entry"
addhelp $a "!learn -l"       "Lock Entry"
addhelp $a "!learn -u"       "Unlock Entry"
}
#################
# Help -End     #
#################

if {![info exists firstload]} {
    add_helps
    add_configs
    if {[catch {load_config $a}]} {
        setup_globals $a
    }
    set_config $a fields "word def author date locked"
    set_config $a dojoin 1
    set firstload "1"
}

proc init {} {
    variable a

    bind pub - ??         KobazLearn::pub_lookup
    bind pub - !learn     KobazLearn::pub_learn
    bind pub - !listkeys  KobazLearn::pub_listkeys

    putlog "${a}.tcl: Loading Learnfile(s)..."

    datafile_loadall $a

    putlog "${a}.tcl by Kobaz Loaded."
    return
}

proc error {args} {
    putlog "LEARN=:=ERROR $args"
}

proc debug {args} {
    variable a
    set debug [get_config $a debug]
    if {$debug == "1"} {
	putlog "LEARN:DEBUG $args"
    }
}
proc add_learn {channel word def author date locked} {
    variable a
    
    set file [get_config ${a}_$channel datafile]

    set learn ""
    lappend learn $word
    lappend learn $def
    lappend learn $author
    lappend learn $date
    lappend learn $locked
    
    dbInsert $file $learn
    datafile_save $a $file

    return [dbNumRecs $file]
}

proc mod_learn {channel fieldname recnum data} {
    variable a

    set file [get_config ${a}_$channel datafile]
    dbModify $file $fieldname $recnum $data

    return
}

proc del_learn {channel recnum} {
    variable a

    set file [get_config ${a}_$channel datafile]
    dbDelete $file $recnum
    return [dbNumRecs $file]
}

proc get_learn {channel word def recnum} {
    variable a
    upvar $def    mydef
    upvar $recnum myrecnum

    set file [get_config ${a}_$channel datafile]

    if {[get_config ${a}_$channel dropcase] == "1"} {
        set word [string tolower $word]
    }
    set num [dbBinarySearch $file word $word]
    if {$num != "-1"} {
	set mydef [dbField $file def $num]
        set myrecnum $num

	return 1;
    }
    
    return 0;
}

# Takes channel and record number
# Writes to var: author
# Writes to var: date
# Returns "locked" if key is locked
# Returns "" if key is not locked
proc get_learninfo {channel recnum author date sincedate} {
    variable a
    upvar $author    myauthor
    upvar $date      mydate
    upvar $sincedate mysincedate

    set file [get_config ${a}_$channel datafile]
    set myauthor    [dbField $file author $recnum]
    set mydate      [dbField $file date   $recnum]
    set mysincedate [lrange [since $mydate] 0 1]

    if {[dbField $file locked $recnum] == "1"} {
	return "locked"
    }

    return
}

#####################
# Binding for ??
#####################
proc pub_lookup {nick uhost handle channel arg} {
    variable a
    set channel [string tolower $channel]
    if {[get_config ${a}_$channel enable] == 0} {return}
    if {[catch {datafile_load_forchannel $a $channel} error]} {putlog $error; return}
    if {![check_access $nick $channel $handle\
	    [get_config ${a}_$channel ??]\
	    [get_config ${a}_$channel msg]]} {
	return
    }

    set type notice

    set arg [string trim $arg " "]
    if {$arg == ""} {
	tellnick $type $nick "Usage: ?? ?cmd? ?nick? <KeyName>"
	return
    }

    # Display help
    if {[string range $arg 0 2] == "-h"} {
        tellnick $type $nick "?? HELP"
        tellnick $type $nick "-h               -help"
        tellnick $type $nick "<key>            -look up <key>"
        tellnick $type $nick "-i <key>         -info on <key>"
        tellnick $type $nick "-t <nick> <key>  -tell <key> to <nick>"
        return
    }

    set cmd  ""
    set word ""
    set tonick ""

    if {[regexp {^(-[^ ]).*$} $arg match cmd]} {
        regexp {^-[^ ] ([^ ]*)$} $arg junk word
        if {[string index $arg 1] == "t"} {
						# 20041121, ASB, handle multiword keys
            #if {![regexp {^-t ([^ ]*) ([^ ]*)$} $arg junk tonick word]} { #}
            if {![regexp {^-t ([^ ]*) (.*)} $arg junk tonick word]} {
                tellnick $type $nick "Usage: ?? -t <nick> <KeyName>"
                return
            }
        }
        if {[string index $arg 1] == "i"} {
						# 20041121, ASB, handle multiword keys
            #if {![regexp {^-i ([^ ]*)$} $arg junk word]} { #}
            if {![regexp {^-i (.*)} $arg junk word]} {
                tellnick $type $nick "Usage: ?? -i <KeyName>"
                return
            }
        }
    } else {
        set word $arg
    }

    if {$word == ""} {
        tellnick $type $nick "Lookup: You must specify a key"
        return
    }
    
		# 20041121, ASB, wrap quotes around lookup if not there already
		if {[string match "* *" $word] && ![regexp {(^".*"$)} $word]} {
        set word "\"$word\""
	  }

    set keylength [string length $word]
    set maxkeylength [get_config ${a}_$channel maxkeylength]
    if {$keylength > $maxkeylength} {
        tellnick $type $nick "Lookup: Key to long."
        return
    }
    
    if {![get_learn $channel $word def recnum]} {
        tellnick $type $nick "Lookup: Non-existant key: $word"
        return
    }

    debug "LEARN: cmd    : $cmd"
    debug "LEARN: word   : $word"
    debug "LEARN: tonick : $tonick"
    debug "LEARN: def    : $def"

    if {$cmd == ""} {
        tellnick $type $channel "$word == $def"
    }

    switch -- $cmd {
        "-i" {
            if {[get_learninfo $channel $recnum author date sincedate] == "locked"} {
                set author "Key is locked"
            }
            
            tellnick $type $channel "$word == $def"
            tellnick $type $channel "Author: $author"
            tellnick $type $channel "Added : $sincedate ago ([clock format $date])"
            return
        }
        
        "-t" {
            if {![onchan $tonick $channel]} {
                tellnick notice $nick "$tonick is not on: $channel"
                return
            }

            tellnick notice $nick "Telling $tonick: $word"
            tellnick notice $tonick "From $nick: $word == $def"
            return
        }
    }


    return
}

proc pub_learn {nick uhost handle channel arg} {
    variable a
    set channel [string tolower $channel]
    if {[get_config ${a}_$channel enable] == 0} {return}
    if {[catch {datafile_load_forchannel $a $channel} error]} {putlog $error; return}

    set type notice
    set arg [string trim $arg " "]

    if {$arg == ""} {
        tellnick $type $nick "Invalid argument and/or usage.  !learn -h for help"
        return
    }

    set cmd ""
    set word ""
    set def ""

    regexp {([^ ]*)} $arg junk cmd
    regexp {([^ ]*) ([^ ]*)} $arg junk cmd word
    regexp {([^ ]*) ([^ ]*) (.*)} $arg junk cmd word def
		# 20041121, ASB, added to handle quoted stuff
    regexp {([^ ]*) ("[^"]*") (.*)} $arg junk cmd word def
    regexp {([^ ]*) ("[^"]*")} $arg junk cmd word

    debug "----------------"
    debug "LEARN: cmd : $cmd"
    debug "LEARN: word: $word"
    debug "LEARN: def : $def"

    if {[get_config ${a}_$channel dropcase] == "1"} {
	set word [string tolower $word]
    } else {
	set word $word
    }

    # This makes -a not required for adding entries
    if {[string index $cmd 0] != "-"} {
        set word $cmd
        set cmd "-a"
        regexp {([^ ]*) (.*)} $arg junk word def
				# 20041121, ASB, added to handle multiword defs w/ quotes
        regexp {("[^"]*") (.*)} $arg junk word def
    }

    if {($word != "") && ($cmd != "")} {
        regsub -all {\\} $word {} newword
        if {$word != $newword} {
            tellnick $type $nick "Learn: Invalid key name"
            return
        }
    }

    switch -- $cmd {
	-h {
            tellnick $type $nick "Learn HELP"
            tellnick $type $nick "-h  -help"

	    if {[check_access $nick $channel $handle\
		    [get_config ${a}_$channel "!learn -a"] ""]} {
                tellnick $type $nick "-a  -add       <key> <defination>"
	    }
	    if {[check_access $nick $channel $handle\
		    [get_config ${a}_$channel "!learn -m"] ""]} {
                tellnick $type $nick "-m  -modify    <key> <defination>"
	    }
	    if {[check_access $nick $channel $handle\
		    [get_config ${a}_$channel "!learn -d"] ""]} {
                tellnick $type $nick "-d  -delete    <key> <defination>"
	    }
	    if {[check_access $nick $channel $handle\
		    [get_config ${a}_$channel "!learn -l"] ""]} {
                tellnick $type $nick "-l  -lock      <key>"
	    }
	    if {[check_access $nick $channel $handle\
		    [get_config ${a}_$channel "!learn -u"] ""]} {
                tellnick $type $nick "-u  -unlock    <key>"		
	    }
            return
	}

	-a {
	    if {![check_access $nick $channel $handle\
		    [get_config ${a}_$channel "!learn -a"]\
		    [get_config ${a}_$channel msg]]} {
		return
	    }

	    set keylength [string length $word]
	    if {$keylength > [get_config ${a}_$channel maxkeylength]} {
		tellnick $type $nick "Key to long."
		return
	    }

	    if {$def == ""} {
		tellnick $type $nick "Usage: !learn ?-a? <key> <defination>"
		return
	    }

	    if {[get_config ${a}_$channel nocolors]} {
		set newword [ridcolor $word]
		set newdef [ridcolor $def]
		if {($newword != $word) || ($newdef != $def)} {
		    set colors "1"
		    set word $newword
		    set def $newdef
		} else {
		    set colors "0"
		}
	    }

            if {[get_learn $channel $word def recnum]} {
                tellnick $type $nick "Learn: Key already exists.  Use -m to modify"
                return
            }

	    set author "${nick}!$uhost"
	    set date   [clock seconds]
	    set locked 0

            set numlearn [add_learn $channel $word $def $author $date $locked]

	    tellnick $type $nick "Learn: $numlearn total. Added Key: $word"
	    if {$colors == "1"} {
		tellnick $type $nick "Learn: Color codes detected in added text, these\
			will be stripped.  In the Future please do not add entries with colors."
	    }

	    return
	}
	
	-m {
            if {![check_access $nick $channel $handle\
                    [get_config ${a}_$channel "!learn -m"]\
                    [get_config ${a}_$channel msg]]} {
                return
            }
   
	    if {$def == ""} {
		tellnick $type $nick "Usage: !learn -m <key> <defination>"
		return
	    }

            if {![get_learn $channel $word "" recnum]} {
                tellnick $type $nick "Learn: Non-existant key: $word"
                return
            }

            if {[get_learninfo $channel $recnum "" "" ""] == "locked"} {
                set locked 1
            } else {
                set locked 0
            }

	    if {($locked == "1") &&\
		    (![check_access $nick $channel $handle\
		    [get_config ${a}_$channel flagoveride]\
                    "Learn: Key locked: $word"])} {
		return
	    }

	    if {[get_config ${a}_$channel nocolors]} {
		set newword [ridcolor $word]
		if {$newword != $word} {
		    set colors "1"
		    set word $newword
		} else {
		    set colors "0"
		}
	    }

            set author "${nick}!$uhost"
            set date   [clock seconds]

            mod_learn $channel def    $recnum $def
            mod_learn $channel author $recnum $author
            mod_learn $channel date   $recnum $date
            tellnick $type $nick "Learn: Key updated: $word"

            if {$colors == "1"} {
                tellnick $type $who "Learn: Color codes detected in added text,\
                      these will be stripped.  In the Future please \
                      do not add entries with colors."
            }
            return
	}

	-d {
            if {![check_access $nick $channel $handle\
                    [get_config ${a}_$channel "!learn -d"]\
                    [get_config ${a}_$channel msg]]} {
                return
            }
    
	    if {$word == ""} {
		tellnick $type $nick "Usage: !learn -d <key>"
		return
	    }

            if {![get_learn $channel $word def recnum]} {
                tellnick $type $nick "Learn: Non-existant key: $word"
                return
            }

            if {[get_learninfo $channel $recnum "" "" ""] == "locked"} {
                set locked 1
            } else {
                set locked 0
            }

	    if {($locked == "1") &&\
		    (![check_access $nick $channel $handle\
		    [get_config ${a}_$channel flagoveride]\
                    "Learn: Key locked: $word"])} {
		return
	    }

            set numlearn [del_learn $channel $recnum]
	    tellnick $type $nick "Learn: $numlearn total. Deleted Key: $word"
	    return
	}
	
	-l {
            if {![check_access $nick $channel $handle\
                    [get_config ${a}_$channel "!learn -l"]\
                    [get_config ${a}_$channel msg]]} {
                return
            }

	    if {$word == ""} {
		tellnick $type $nick "Usage: !learn -l <key>"
		return
	    }


            if {![get_learn $channel $word def recnum]} {
                tellnick $type $nick "Learn: Non-existant key: $word"
                return
            }

            if {[get_learninfo $channel $recnum author date ""] == "locked"} {
		tellnick $type $nick "Key: Already locked: $word"
                return
            }
            
            debug "LOCK: Locking learn: $word"
            mod_learn $channel locked $recnum 1
            tellnick $type $nick "Learn: Locked: $word"

            return
	}
	-u {
            if {![check_access $nick $channel $handle\
                    [get_config ${a}_$channel "!learn -u"]\
                    [get_config ${a}_$channel msg]]} {
                return
            }

	    if {$word == ""} {
		tellnick $type $nick "Usage: !learn -u <key>"
		return
	    }


            if {![get_learn $channel $word def recnum]} {
                tellnick $type $nick "Learn: Non-existant key: $word"
                return
            }

            if {[get_learninfo $channel $recnum author date ""] == ""} {
		tellnick $type $nick "Key: Not locked: $word"
                return
            }
            
            debug "LOCK: UnLocking learn: $word"

            mod_learn $channel locked $recnum 0

            tellnick $type $nick "Learn: Unlocked: $word"
            return
	}
    }

    tellnick $type $nick "Invalid argument and/or usage.  !learn -h for help"
    return	
}

proc pub_listkeys {nick uhost handle channel arg} {
    variable a
    set channel [string tolower $channel]
    if {[get_config ${a}_$channel enable] == 0} {return}
    if {[catch {datafile_load_forchannel $a $channel} error]} {putlog $error; return}
    if {![check_access $nick $channel $handle\
	    [get_config ${a}_$channel !listkeys]\
	    [get_config ${a}_$channel msg]]} {
	return
    }

    set type notice
    set displaymax [get_config ${a}_$channel displaymax]
    set file [get_config ${a}_$channel datafile]
    
    if {![regexp {^([^ ]*)$} $arg junk search]} {
	tellnick $type $nick "Listkeys: Keys can only be one word"
        return
    }

    if {[get_config ${a}_$channel dropcase] == "1"} {
	set search [string tolower $arg]
    }

    if {$search == ""} {
	tellnick $type $nick "Usage: !listkeys <search>"
	return
    }

    set found [dbMatchingRecords $file word $search]
    debug "LISTKEYS: Search: $search"
    debug "LISTKEYS: Found recs: $found"

    set matches [llength $found]
    if {$matches == 0} {
	tellnick $type $nick "Listkeys: No matches for: $search"
	set maxreached 0
	return
    }

    if {$matches >= $displaymax} {
	if {$matches == 1} {
	    return
	}
	set found [lrange $found 0 $displaymax]
	set maxreached "1"
    } else {
	set maxreached "0"
    }

    set foundlist ""
    foreach recnum $found {
	set foundlist "$foundlist [dbField $file word $recnum]"
    }
    
    if {$maxreached == "1"} {
	set foundlist "$foundlist ...too many matches. Total: $matches"
    }

    debug "LISTKEYS: Named found list: $foundlist"

    tellnick $type $channel "Matching:$foundlist"

    return
}

##########################################
########### End of Major Code ############
##########################################

init

}
