#############################################################################
## Quote Managment System 2.2.4
## KobazQuote.tcl
## 
## AUTHOR: Kobaz Mi'Daire <krobaz@yahoo.com>
## 
## REQUIRES: Eggdrop 1.3.x AND tcl 8.1
##           KobazDatabase.tcl
##           KobazDatabaseAPI.tcl
##           KobazSQL-Lib.tcl
##           KobazConfig.tcl
##           KobazTools.tcl
##           KobazHelp.tcl
##
## PURPOSE: There are many different quote TCLs all over the web, but the
##          problem is, not one of them is totaly complete. I doubt this TCL
##          is complete, but it has more features combined into one script
##          then all the quote TCLs features combined.
##
## FEATURES: Ability to add quotes.
##           Ability to remove quotes.
##           Ability to recover accidently deleted quotes.
##           Ability to search through quotes.
##           Ability to say a random quote at a certan interval.
##           Ability to display a random quote matching a search pattern.
##           Ability to "lock" a quote and prevent its deletion.
##           Ability to "unlock" a quote from the "locked" state, and allow deletion.
##           Ability to view when a quote was added, and by who.
##           Completely Customizeable via dcc.
##           Completely MultiChannel
## 
## COMMANDS: Arguments in "< >" are mandatory.
##           Arguments in "? ?" are optional.  
##           Arguments in "[ ]" are a toggle, and mandatory.
##           Arguments in "( )" must be used together
## 
## !addquote <quote>            - Adds <quote> to the quote list.
## !delquote <quotenum>         - Deletes quote number <quotenum> from the list.
## !quotebucket <args>          - Ability to recover accidently deleted quotes.
## !quote                       - Displays a random quote.
## !quote ?search?              - Displays quotes containing ?search?
## !quote ?(startnum) (search)? - If the search is to large, this is used to 
##                                display the next range of quotes 
##                                matching (search) starting  from (startnum),
##                                by default (startnum) is "0"
## !rquote ?num? <search>       - Randomly displays a quote matching <search>
##                                if ?num? is specified, then ?num? random
##                                quotes will be displayed
## !next (in development)       - Displays next *maxdisplay* matches for a
##       (not enabled)            !quote search
##                                Or, next *maxdisplay* random quotes for a
##                                !rquote search
## !info <quotenum>             - Displays info on <quotenum> including: the 
##                                quote, the author of the quote, and how long 
##                                ago it was added.
## !quotelock [on/off] <num>    - Will lock or unlock a quote in the quote list.
##                                Locking described in -FEATURES-
##
###############################################################################

###############################################################################
# All Configuration is done via the bot now: the .kbconfig KobazQuote command
# If quote commands do not work in a specific channel, remember to use
# .kbconfig KobazQuote to set enable to 1 for that channel
###############################################################################

############################################
############################################
## Modify contents below at your own risk ##
############################################
############################################

source scripts/kobazscripts/KobazDep.tcl
kbrequiretcl 8.1

namespace eval KobazQuote {
set a KobazQuote  

####
# Load required libraries

kbrequire DatabaseAPI
kbrequire Config
kbrequire Tools
kbrequire Help

# end
####

namespace import -force ::KobazTools::*
namespace import -force ::KobazHelp::*
namespace import -force ::KobazConfig::*
namespace import -force ::KobazDBAPI::*

############################
# Settings - Do NOT Modify #
############################
proc add_configs {} {
variable a
add_config $a CREATE . . .
add_config $a "tog global" debug "0" {Enable debug? You probebly really don't need to turn this on}

add_config $a "tog" enable "0" {Enable quote commands on this channel?}
add_config $a "num_no0" displaymax "5" {Maximux quotes to display at once when multiple quotes are displayed}
add_config $a "tog" sherrors "0" {Show detailed errors for public commands?}
add_config $a "num_0" timer_int "10" {Delay in minutes for bot to say a random quote. Set to 0 to disable.}
add_config $a "num_0" quotedelay "10" {Delay in seconds that !quote and similar commands can be used.  Set to 0 to disable}
add_config $a "flag" quotedelayflag "v" {Flags of users who are not affected by quotedelay}
add_config $a "tog" nocolors "1"  {Remove colors of added quotes?}
add_config $a "tog" color_notify "1" {Notify people if colors arent allowed?}
#add_config $a "num_no0" maxcache "20" {Currently not in use}
add_config $a "num_no0" maxbucket "20" {Max quotes in the quote "recycle bin".}
add_config $a "num_no0" bucketexpire "1" {Time in hours for a quote in the "recycle bin" to be considered old and purged}
add_config $a "tog" usebucket "1" {Use quotebucket?}
add_config $a "tog" randomdelay  "1" {Reset random quote timer after someone does a !rquote or !quote}
add_config $a "text" datafile "KobazQuote.db" {Data file to use for this channel}
add_config $a flag !quote                "-"  "Flags to access !quote"
add_config $a flag !rquote               "-"  "Flags to access !rquote"
add_config $a flag !next                 "-"  "Flags to access !next"
add_config $a flag !info                 "-"  "Flags to access !info"
add_config $a flag !match                "-"  "Flags to access !match"
add_config $a flag !addquote             "v"  "Flags to access !addquote"
add_config $a flag !delquote             "o"  "Flags to access !delquote"
add_config $a flag !quotebucket          "o"  "Flags to access !quotebucket"
add_config $a flag "!quotebucket -purge" "cm" "Flags to access !quotebucket -purge"
add_config $a flag !quotelock            "cm" "Flags to access !quotelock"
add_config $a text_star msg "No Access." {Message to give person if denied access to a command. If value is * then no message will be given.}
}
#################
# Settings -End #
#################

#################
# Help          #
#################
proc add_helps {} {
variable a
addhelp $a "!quote"              "Display quote"
addhelp $a "!rquote"             "Display random quotes"
addhelp $a "!info"               "Information on a quote"
addhelp $a "!addquote"           "Adds a quote"
addhelp $a "!delquote"           "Deletes a quote"
addhelp $a "!quotelock"          "Lock/Unlock a quote"
addhelp $a "!quotebucket"        "Recover deleted quote"
addhelp $a "!quotebucket -purge" "Clear quotebucket"

addhelp_extra $a "!quote" "Will display a random quote -\
!quote <string> - will search quotes for the string specified -\
!quote <#n> <string> - will display matches of string starting with match #n"
addhelp_extra $a "!rquote" "Will display random quotes -\
!rquote <string> - will display a random quote matching <string> -\
!rquote <#n> <string> - will display #n number of random quotes matching <string>\
as long as #n isn't over the max display limit" 
}
#################
# Help -End     #
#################

if {![info exists firstload]} {
    #    set cachelist      ""
    #    set cachefirst     ""
    #    set cache(.)       ""
    add_helps
    add_configs
    if {[catch {load_config $a}]} {
        setup_globals $a
    }
    set_config $a fields "quote author date locked"
    set_config $a dojoin 1
    set firstload "1"
}

proc init {} {
    variable a

    bind pub - !addquote    KobazQuote::pub_addquote
    bind pub - !delquote    KobazQuote::pub_delquote
    bind pub - !quote       KobazQuote::pub_quote
    bind pub - !rquote      KobazQuote::pub_rquote
    bind pub - !next        KobazQuote::pub_next
    bind pub - !quotebucket KobazQuote::pub_quotebucket
    bind pub - !info        KobazQuote::pub_info
    bind pub - !quotelock   KobazQuote::pub_lock
    bind pub - !match       KobazQuote::pub_match

    putlog "${a}.tcl: Loading Quotefile(s)..."
    datafile_loadall $a

    putlog "${a}.tcl by Kobaz Loaded."
    return
}

proc error {args} {
    putlog "QUOTE=:=ERROR $args"
}

proc debug {args} {
    variable a
    set debug [get_config $a debug]
    
    if {$debug == "1"} {
	putlog "QUOTE:DEBUG $args"
    }
}

proc check_quotedelay {nick handle channel} {
    variable a

    set quotedelay     [get_config ${a}_$channel quotedelay]
    set quotedelayflag [get_config ${a}_$channel quotedelayflag] 
    if {(![check_access $nick $channel $handle [get_config ${a}_$channel quotedelayflag] ""]) &&\
            ([get_config ${a}_$channel quotedelay] != "0")} {
	set quotetimer [utimerexists "KobazQuote::quotedelay $channel"]
	set timer      [lsearch [utimers] "*$quotetimer"]
	set quotesec   [lindex [lindex [utimers] $timer] 0]
	if {$quotetimer != ""} {
	    tellnick notice $nick "Quote: Busy, please wait $quotesec seconds."
	    return 1
	}
	utimer $quotedelay "KobazQuote::quotedelay $channel"
    }
    return 0
}


proc quotedelay {args} {return}
proc delquote_timer {args} {return}

proc _datafile_load {file channel} {
    variable a
    variable data

    set channel [string tolower $channel]
    if {[get_config ${a}_$channel enable] != "1"} {return}

    if {![info exists data(LastRanQuote,$channel)]}   {set data(LastRanQuote,$channel)   ""}
    if {![info exists data(LastAddedQuote,$channel)]} {set data(LastAddedQuote,$channel) ""}
    if {![info exists data(LastDelQuote,$channel)]}   {set data(LastDelQuote,$channel)   ""}

    set timer_int [get_config ${a}_$channel timer_int]
    if {$timer_int != "0"} {
        set timername [timerexists "KobazQuote::random_quote_pub $channel"]	    
        if {$timername == ""} {KobazQuote::random_quote_pub $channel noload}
        
        debug "RANDOMQUOTE: TimerName for $channel: $timername"
    }
    
    if {![dbLoaded $file.QuoteBucket]} {
        if {![check_exists $file.QuoteBucket]} {
            putlog "${a}.tcl: File $file.QuoteBucket did not exist, creating."
            dbCreate $file.QuoteBucket $file.QuoteBucket {quote author date locked bucket_time bucket_id whodel}
            dbSort $file.QuoteBucket quote
            dbWrite $file.QuoteBucket
        } else {
            if {[catch {dbRead $file.QuoteBucket $file.QuoteBucket} error]} {
                error "Unable to load quotebucket file: $error"
                return
            } else {
                debug "LOADQUOTE: Read quotebucket database: $file.QuoteBucket"
            }
        }
    }

    set numqbkt [dbNumRecs $file.QuoteBucket]
    if {$numqbkt != "0"} {
	set timername [timerexists "KobazQuote::quotebucket_expire $file $channel"]
	if {$timername == ""} {timer 10 "KobazQuote::quotebucket_expire $file $channel"}
    }

    return
}

proc _datafile_save {file {sort ""}} {
    dbSort $file.QuoteBucket bucket_time -decreasing
    debug "SAVEQUOTE: Sorted quotebucket database: $file.QuoteBucket"

    dbWrite $file.QuoteBucket
    debug "SAVEQUOTE: Wrote quotebucket database: $file.QuoteBucket"
    
    return
}

proc _datafile_expire {file} {
    catch {dbClose $file.QuoteBucket}
    return
}

proc _datafile_pub_join {channel} {
    variable a

    set channel [string tolower $channel]
    if {[get_config ${a}_$channel enable] != "1"} {return}

    set timer_int [get_config ${a}_$channel timer_int]
    if {$timer_int != "0"} {
        set timername [timerexists "KobazQuote::random_quote_pub $channel"]	    
        if {$timername == ""} {KobazQuote::random_quote_pub $channel noload}
        
        debug "RANDOMQUOTE: TimerName for $channel: $timername"
    }
    return
}

proc random_quote {file {search ""}} {
    set numquotes [dbNumRecs $file]

    if {$search != ""} {
	set found [dbMatchingRecords $file $file *${search}*]
	set randquotenum [lindex $found [random [llength $found]]]
    } else {
	set randquotenum [random $numquotes]
    }
    
    debug "RANQUOTE: Random number out of: $numquotes is: $randquotenum"
    return $randquotenum
}

proc random_quote_pub {channel {noload ""}} {
    variable a
    variable data

    set channel [string tolower $channel]
    set timer_int [get_config ${a}_$channel timer_int]
    if {([get_config ${a}_$channel enable] == "0") || ($timer_int == "0")} {return}

    if {$noload != "noload"} {
        if {[catch {datafile_load_forchannel $a $channel} error]} {putlog $error; return}
    }

    if {![botonchan $channel]} {return}

    set file [get_config ${a}_$channel datafile]

    if {$file == ""} {return}
    if {[dbNumRecs $file] == 0} {return}

    set foundquote [random_quote $file]
    debug "TIMERQUOTE: Quote index: $foundquote"

    set data(LastRanQuote,$channel) [dbField $file quote $foundquote]
    set foundquote [dbField $file quote $foundquote]

    if {[get_config ${a}_$channel nocolors]} {
	set foundquote [ridcolor $foundquote]
    }

    debug "TIMERQUOTE: $channel: Quote would be: $foundquote"
    tellnick pub $channel "$foundquote"

    set timername [timerexists "KobazQuote::random_quote_pub $channel"]
    if {$timername == ""} {timer $timer_int "KobazQuote::random_quote_pub $channel"}

    return
}

proc random_quote_reset {channel} {
    variable a

    set channel [string tolower $channel]

    if {[get_config ${a}_$channel randomdelay] == "0"} {return}

    set timer_int [get_config ${a}_$channel timer_int]

    set timername [timerexists "KobazQuote::random_quote_pub $channel"]
    if {$timername != ""} {killtimer $timername}

    timer $timer_int "KobazQuote::random_quote_pub $channel"

    return
}

proc display_quotes {file type channel found {nonum ""}} {
    variable a

    set displaymax [get_config ${a}_$channel displaymax]
    set channel [string tolower $channel]

    debug "QUOTE: $channel: Displaying indexes: $found"
    
    set count 0
    foreach temp $found {
	if {$count > $displaymax} {break}
	set quote [dbField $file quote $temp]
	if {[get_config ${a}_$channel nocolors]} {
	    set quote [ridcolor $quote]
	}
        if {$nonum == "nonum"} {
            tellnick $type $channel "$quote"
        } else {
            tellnick $type $channel "${temp}. $quote"
        }
	incr count
    }
    return
}

proc display_quotebucket {file type channel found} {
    variable a

    set channel [string tolower $channel]
    set displaymax [get_config ${a}_$channel displaymax]
    debug "QUOTEBUCKET: Displaying indexes: $found"
    
    set count 0
    foreach temp $found {
	if {$count > $displaymax} {break}
	set quote [dbField $file.QuoteBucket quote $temp absolute]
	if {[get_config ${a}_$channel nocolors]} {set quote [ridcolor $quote]}
	tellnick $type $channel "${temp}. $quote"
	incr count
    }
    return
}

proc quotebucket_expire {file channel} {
    variable a

    set channel      [string tolower $channel]
    set bucketexpire [get_config ${a}_$channel bucketexpire]
    set numqbkt      [dbNumRecs $file.QuoteBucket]
    
    set del_list ""

    for {set x 0} {$x < $numqbkt} {incr x} {
	set time  [dbField $file.QuoteBucket bucket_time $x absolute]
        if {$time == ""} {continue}

        set curtime [clock seconds]
        set howlong [expr $curtime - $time]
        set hours [expr ($howlong / 60) / 60]
	if {$hours >= $bucketexpire} {
	    lappend del_list $x
	}
    }

    if {$del_list != ""} {
	set del_list [revorder $del_list]
	debug "QBKTEXPIRE: $file.QuoteBucket: Found old quotes, expireing from quotebucket"
	debug "QBKTEXPIRE: $file.QuoteBucket: ExpireList: $del_list"
	foreach quotenum $del_list {
	    dbDelete $file.QuoteBucket $quotenum
	}
	dbWrite $file.QuoteBucket
    }
    
    set numqbkt [dbNumRecs $file.QuoteBucket]
    if {$numqbkt != "0"} {
	set timername [timerexists "KobazQuote::quotebucket_expire $file $channel"]
	if {$timername == ""} {timer 1 "KobazQuote::quotebucket_expire $file $channel"}
    }
    
    return
}

proc pub_addquote {nick uhost handle channel arg} {
    variable a
    set channel [string tolower $channel]
    if {[get_config ${a}_$channel enable] == 0} {return}
    if {[catch {datafile_load_forchannel $a $channel} error]} {putlog $error; return}
    if {![check_access $nick $channel $handle\
	    [get_config ${a}_$channel !addquote]\
	    [get_config ${a}_$channel msg]]} {
	return
    }
    
    variable data
    set type "notice"
    set errortype "notice"
    set who $nick
    
    set file [get_config ${a}_$channel datafile]

    debug "---------------"
    
    if {$arg == ""} {
	debug "ADDQUOTE: Not enough args"
	tellnick $type $who "Usage: !addquote quote"
	return
    }
    
    set author "${nick}!$uhost"
    set date [clock seconds]
    
    if {[get_config ${a}_$channel nocolors]} {
	set newarg [ridcolor $arg]
	if {$newarg != $arg} {
	    set colors "1"
	} else {
	    set colors "0"
	    set arg $newarg
	}
    }

    set searchquote $arg
    
    debug "ADDQUOTE: $channel: Added line will be: $arg"
    debug "ADDQUOTE: $channel: Searching if quote exists"
    
    set found [dbBinarySearch $file quote $searchquote]
    
    if {$found == "-1"} {
	lappend quote $arg
	lappend quote $author
	lappend quote $date
	lappend quote 0
	set result [dbInsert $file $quote]
        if {$result != "0"} {
            tellnick $type $who "Addquote: failed."
            error "ADDQUOTE: $channel: addquote failed: $result"
            return
        }
	
	debug "ADDQUOTE: $channel: Added quote $arg"
	debug "ADDQUOTE: $channel: Set author of quote to $author"
	debug "ADDQUOTE: $channel: Set time/date of quote to $date"
	debug "ADDQUOTE: $channel: Number of records: [dbNumRecs $file]"
	
	tellnick $type $who "Addquote: Quote added. [dbNumRecs $file] total quotes."
	if {$colors == "1" && [get_config ${a}_$channel color_notify] == "1"} {
	    tellnick $type $who "Addquote: Color codes detected in added quote,\
		    these will be stripped.  In the Future please do not add\
		    quotes with colors."
	}
	datafile_save $a $file
	set data(LastAddedQuote,$channel) $searchquote
    } else {
	debug "ADDQUOTE: Quote exists"
	tellnick $type $who "Addquote: Quote already exists: (#${found})"
	return
    }
    return
}

proc pub_delquote {nick uhost handle channel arg} {
    variable a

    set channel [string tolower $channel]
    if {[get_config ${a}_$channel enable] == 0} {return}    
    if {[catch {datafile_load_forchannel $a $channel} error]} {putlog $error; return}
    if {![check_access $nick $channel $handle\
	    [get_config ${a}_$channel !delquote]\
	    [get_config ${a}_$channel msg]]} {
	return
    }

    variable data

    set maxbucket [get_config ${a}_$channel maxbucket]
    set file      [get_config ${a}_$channel datafile]

    set type "pub"
    set errortype "notice"
    set who $nick
    
    set remove_quote [lindex $arg 0]
    
    regsub -all {[0-9]} $arg "" nonum
    if {$nonum != ""} {
	tellnick $errortype $nick "Delquote: Argument must be a number."
	return
    }
    
    if {$arg == ""} {
	tellnick $errortype $who "Delquote: Usage: !delquote quotenum" 

	return
    }

    if {[dbNumRecs $file] == 0} {
	tellnick $errortype $who "There are no quotes."
	return
    }

    set quote [dbField $file quote $arg]
    if {$quote == ""} {
        tellnick $errortype $who "Delquote: Quote number $arg dosn't exist."
        return
    }

    set delquote_timer [utimerexists "KobazQuote::delquote_timer $channel"]
    set timer        [lsearch [utimers] "*KobazQuote::delquote_timer $channel*"]
    set delquote_sec [lindex [lindex [utimers] $timer] 0]
    if {$data(LastDelQuote,$channel) == $remove_quote} {
	debug "DELQUOTE: $channel: Last deleted quote is the same as this one, suspicious"
	if {$delquote_timer != ""} {
	    debug "DELQUOTE: $channel: Lag protection on quote $remove_quote. Did not trash"
	    tellnick $errortype $who "Delquote: Lag protection.\
		    Cannot trash quote ${remove_quote}. It will be\
		    $delquote_sec seconds before you can remove this\
		    quote."
	    return
	}
    }
    
    set locked [dbField $file locked $remove_quote]
    if {$locked == 1} {
	tellnick $errortype $who "Delquote: Cannot trash entry ${remove_quote}, Reason: Locked"
	return
    }

    set curtime [clock seconds]
    
    if {[get_config $a usebucket] == "1"} {
        set numrecs [dbNumRecs $file.QuoteBucket]
        if {$numrecs != 0} {
            for {set x 0} {$x < $numrecs} {incr x} {
                lappend ids [dbField $file.QuoteBucket bucket_id $x absolute]
            }
            set ids [lsort $ids]
        } else {
            set ids 0
            while {$ids < 400} {
                set ids [random 2000]
            }
        }
        
        set bucket_id [expr [lindex $ids end] + 1]
    
        set add ""
        lappend add [dbField $file quote  $remove_quote]
        lappend add [dbField $file author $remove_quote]
        lappend add [dbField $file date   $remove_quote]
        lappend add [dbField $file locked $remove_quote]
        lappend add $curtime
        lappend add $bucket_id
        lappend add "${nick}!$uhost"
        dbInsert $file.QuoteBucket $add
    }

    dbDelete $file $remove_quote
    datafile_save $a $file

    debug "DELQUOTE: $channel: Deleting/Trashing quote number $arg"

    if {[get_config $a usebucket] == "1"} {
        debug "DELQUOTE: $channel: Trashing quote number $arg"
        tellnick $errortype $who "Delquote: Trashed quote $remove_quote.  [dbNumRecs $file] total quotes.\
                If you trashed this quote in error type !quotebucket -restore $bucket_id"
    } else {
        tellnick $errortype $who "Delquote: Deleted quote $remove_quote.  [dbNumRecs $file] total quotes."
    }

    debug "DELQUOTE: [dbNumRecs $file] total quotes now"

    set LastAddedQuoteNum [dbBinarySearch $file quote $data(LastAddedQuote,$channel)]
    if {$remove_quote == $LastAddedQuoteNum} {    
	set LastAddedQuote "-1"
    }
    if {$remove_quote == $data(LastRanQuote,$channel)} {
	set LastRanQuote($channel) "-1"
    }
    
    set data(LastDelQuote,$channel) $remove_quote
    
    set delquote_timer [utimerexists "KobazQuote::delquote_timer $channel"]
    catch {killutimer $delquote_timer}
    utimer 10 "KobazQuote::delquote_timer $channel"

    if {[get_config $a usebucket] == "1"} {    
        set timername [timerexists "KobazQuote::quotebucket_expire $file $channel"]
        if {$timername == ""} {timer 1 "KobazQuote::quotebucket_expire $file $channel"}
    }

    return
}

proc pub_quote {nick uhost handle channel {arg ""}} {
    variable a
    set channel [string tolower $channel]
    if {[get_config ${a}_$channel enable] == 0} {return}
    if {[catch {datafile_load_forchannel $a $channel} error]} {putlog $error; return}
    if {![check_access $nick $channel $handle\
	    [get_config ${a}_$channel !quote]\
	    [get_config ${a}_$channel msg]]} {
	return
    }

    set file [get_config ${a}_$channel datafile]

    variable data
    set type "pub"
    set errortype "notice"
    set who $nick
    set argz $arg

    if {[check_quotedelay $nick $handle $channel]} {return}

    set sherrors   [get_config ${a}_$channel sherrors]
    set displaymax [get_config ${a}_$channel displaymax]
    
    debug "---------------"
    debug "QUOTE: $channel: ARGS: $arg"
    
    if {[dbNumRecs $file] == 0} {
	tellnick $errortype $who "There are no quotes."
	return
    }

    if {$arg == ""} {
	debug "QUOTE: $channel: No args, getting random quote."
	set foundquote [random_quote $file]
	debug "QUOTE: $channel: Random quote index will be: $foundquote"
        set data(LastRanQuote,$channel) [dbField $file quote $foundquote]
	display_quotes $file $type $channel $foundquote nonum
        random_quote_reset $channel

	return
    }

    set firstarg [lindex $arg 0]

    switch -- $firstarg {
	-last {
	    debug "QUOTE: $channel: User requested last randomly displayed quote."
	    if {$data(LastRanQuote,$channel) == ""} {
		tellnick $errortype $nick "Quote: No random quote said yet."
		return
	    }
	    set LastRanQuoteNum [dbBinarySearch $file quote $data(LastRanQuote,$channel)]
	    if {$LastRanQuoteNum == "-1"} {
		tellnick $errortype $nick "Quote: Last random quote was deleted."
		return
	    }
	    display_quotes $file $type $channel $LastRanQuoteNum
            random_quote_reset $channel
	    return
	}
	
	-lastadded {
	    debug "QUOTE: $channel: User requested last added quote."
	    if {[llength $data(LastAddedQuote,$channel)] == "0"} {
		tellnick $errortype $nick "Quote: Last Added Quote was not recorded."
		return
	    }   
	    set LastAddedQuoteNum [dbBinarySearch $file quote $data(LastAddedQuote,$channel)]
	    if {$LastAddedQuoteNum == "-1"} {
		tellnick $errortype $nick "Quote: Last Added Quote was Deleted."
		return
	    }
	    display_quotes $file $type $channel $LastAddedQuoteNum
            random_quote_reset $channel
	    return
	}

	-any {
	    debug "QUOTE: $channel: User requested random $displaymax quotes."
	    
	    set found ""
	    
	    for {set x 0} {$x < $displaymax} {incr x} {
		set quote [random_quote $file]
		if {[lsearch -exact $found $quote] == "-1"} {
		    lappend found $quote
		} else {
		    while {[lsearch -exact $found $quote] != "-1"} {
			set quote [random_quote $file]
		    }
		    lappend found $quote
		}
	    }

	    debug "QUOTE: $channel: Displaying quotes $found"
	    tellnick $type $channel "Displaying ${displaymax} random quotes."
	    display_quotes $file $type $channel $found
            random_quote_reset $channel
	    return
	}

	-ss {
	    return
	    debug "QUOTE: User requested Kano's SuperSearch(tm)"
	    if {$realsearch == ""} {
		tellnick $errortype $nick "Usage: !quote -ss <options>"
		return
	    }
	    start_bench
	    set found [parseterms "$realsearch" 0 $numquotes]
	    set found [lindex $found 1]
	    set ms [stop_bench]	
	    set arg $arg2
	}

	-help {
	    if {$arg3 == "-ss"} {
                return
		tellnick $errortype $who "-ss        - Searches quotes with supersearch"
		tellnick $errortype $who "Uses a search similar to altavista.  You can specify"
		tellnick $errortype $who "things that must be displayed with: +.  And also can"
		tellnick $errortype $who "display things that must not be displayed with: -"
		return
	    }
	    tellnick $errortype $who "!quote Arguments:"
	    tellnick $errortype $who "-help      - Display help (this text)"
#	    tellnick $errortype $who "-ss        - Searches quotes with supersearch: see !quote -help -ss"
	    tellnick $errortype $who "-any       - Display any 5 random quotes"
	    tellnick $errortype $who "-last      - Display the last randomly displayed quote"
	    tellnick $errortype $who "-lastadded - Display the last added quote"
	    tellnick $errortype $who "KobazQuote.tcl written and directed by Kobaz"
	    return
	}
    }

    if {$arg != ""} {
	set startquote [lindex $arg 0]

	regsub -all {[0-9]} $startquote "" startquote_nonum
	if {$startquote_nonum == "" && $startquote != ""} {
	    set arg [join [lrange [split $arg] 1 end]]
	} else {
	    set startquote 0
	}
	
	if {$startquote_nonum == "" && $arg == ""} {
	    set requested_one_quote 1
	}	    
	unset startquote_nonum

	set searchquote $arg
	regsub -all {\\|\[|\]} $searchquote {\\&} realsearch

	if {[info exists requested_one_quote] == "1"} {
            set quote [dbField $file quote $startquote]
            if {$quote == ""} {
		debug "QUOTE: $channel: User requested non-existant quote"
		tellnick $errortype $who "Quote: There is no quote number: $argz"
		return
	    }
	    debug "QUOTE: $channel: User requested of displaying one quote"
	    display_quotes $file $type $channel $startquote nonum
            random_quote_reset $channel
	    return
	}
	
	debug "QUOTE: $channel: Search is: $searchquote"
	if {![info exists found]} {
	    set listargs [split $realsearch ""]
	    set numwild 0
	    foreach argitem $listargs {
		if {[regexp {\*} $argitem] == "1"} {incr numwild}
		if {[regexp {\?} $argitem] == "1"} {incr numwild}
	    }
	    if {$numwild >= "5"} {
		debug "QUOTE: $channel: Too many wildcards."
		tellnick $errortype $nick "Quote: Too many wildcards."
		return
	    }
	    debug "QUOTE: $channel: Quote starting from: $startquote"

	    #####
	    #####
	    #####
	    
	    if {1 == 0} {	    
		set cached 0
		debug "QUOTE: Searches Cached: $KobazQuote::cachelist"
		if {[lsearch $KobazQuote::cachelist $arg] != "-1"} {
		    debug "QUOTE: Search was cached (phew)"
		    set found $KobazQuote::cache($arg)
		    set ms {[cached]}
		    set cached 1
		}
		
		if {$cached == 0} {
		    debug "QUOTE: Search not cached, cacheing"
		    start_bench
		    set found [dbMatchingRecords quotes quote *${arg}*]
		    set ms [stop_bench]
		    
		    set KobazQuote::cache($arg)   $found
		    lappend KobazQuote::cachelist $arg
		    set maxcache [get_config $a maxcache]
		    if {[llength $KobazQuote::cachelist] > $maxcache} {
			debug "QUOTE: Cache too large, removing one entry"
			set KobazQuote::cachefirst [lindex $KobazQuote::cachelist 0]
			unset KobazQuote::cache($KobazQuote::cachefirst)
			set KobazQuote::cachelist [lreplace $KobazQuote::cachelist 0 0]
			debug "QUOTE: Searches Cached: $KobazQuote::cachelist"
		    }
		}
		set ms ":Q:"
	    }

	    #####
	    #####
	    #####
	    
	    start_bench
	    set found [dbMatchingRecords $file quote *${realsearch}*]
	    set ms [stop_bench]
	    
	    set numfound [llength $found]
	    debug "QUOTE: $channel: Number of matches: $numfound"
	    
	    if {([string length $realsearch] < 2) && ($numfound > "300")} {
		tellnick $type $channel "$ms Search too large.  Total matches $numfound"
		return
	    }
	    
	    debug "QUOTE: $channel: Maximum to display at once: $displaymax"

	    if {$found == ""} {
		debug "QUOTE: $channel: No quotes found matching search"
		tellnick $errortype $who "Quote: No quotes matching \"${searchquote}\""
		return
	    } else {
		if {$startquote >= $numfound} {
		    debug "QUOTE: $channel: User requested more quotes than found"
		    tellnick $errortype $who "Quote: There are no more quotes matching\
			    \"${searchquote}\""
		    return
		}

		set firstnumber $startquote
		set secondnumber [expr $startquote + $displaymax -1]
		
		set displayone [expr $firstnumber +1]
		set displaytwo [expr $secondnumber +1]
		set lastnum [llength $found]
		debug "QUOTE: $channel: Last num:  $lastnum"
		
		debug "QUOTE: $channel: First Num: $firstnumber"
		debug "QUOTE: $channel: Secnd Num: $secondnumber"
		debug "QUOTE: $channel: DisplayingOne: $displayone"
		debug "QUOTE: $channel: DisplayingTwo: $displaytwo"
		
		if {$numfound == 1} {
		    set s ""
		    set areis is
		} else {
		    set s "s"
		    set areis are
		}
		
		if {[expr $firstnumber +1] >= $numfound} {
		    set s ""
		    set areis is
		}
		
		set found [lrange $found $startquote $secondnumber]
		
		if {[expr $startquote +1] > $numfound} {
		    debug "QUOTE: $channel: Start number is greater than total matches"
		}
		
		set start 0
		set message  "$ms ${numfound} quote$s matching \"${searchquote}\". Displaying quote$s:"
		set message2 "To display the next $displaymax type: !quote [expr $secondnumber +1] $searchquote"
		
		if {$numfound == "1"} { 
		    set start 1
		    debug "QUOTE: $channel: One quote found."
		    set displaynum "${displayone}"
		    tellnick $type $channel "$message ${displaynum}."
		}
		
		if {$numfound <= $displaymax && $start == "0"} { 
		    set start 1
		    debug "QUOTE: $channel: Less than maximum display number"
		    set displaynum "${displayone}-${lastnum}"
		    tellnick $type $channel "$message ${displaynum}."
		}
		
		if {$displayone == $lastnum && $start == "0"} {
		    set start 1
		    debug "QUOTE: $channel: User asked for the last quote."
		    set displaynum "${lastnum}"
		    tellnick $type $channel "$message ${displaynum}."
		}		
		
		if {$displaytwo >= $lastnum && $start == "0"} { 
		    set start 1
		    debug "QUOTE: $channel: End of quote list, but more than one displaying"
		    set displaynum "${displayone}-${lastnum}"
		    tellnick $type $channel "$message ${displaynum}. $message2"
		}
		
		if {$secondnumber < $lastnum && $start == "0"} {
		    debug "QUOTE: $channel: To many matches to display, only displaying $displaymax quote$s"
		    set displaynum "${displayone}-${displaytwo}"
		    tellnick $type $channel "$message ${displaynum}. $message2"
		}
		
		display_quotes $file $type $channel $found
                random_quote_reset $channel
		return  
	    }
	}
    }
}

proc pub_rquote {nick uhost handle channel arg} {
    variable a
    set channel [string tolower $channel]
    if {[get_config ${a}_$channel enable] == 0} {return}    
    if {[catch {datafile_load_forchannel $a $channel} error]} {putlog $error; return}
    if {![check_access $nick $channel $handle\
	    [get_config ${a}_$channel !rquote]\
	    [get_config ${a}_$channel msg]]} {
	return
    }

    set displaymax [get_config ${a}_$channel displaymax]
    set file       [get_config ${a}_$channel datafile]

    variable numquotes
    set type "pub"
    set errortype "notice"
    set who $nick

    if {[check_quotedelay $nick $handle $channel]} {return}
    
    debug "---------------"
    debug "RQUOTE: ARGS: $arg"
    debug "RQUOTE: CHANNEL: $channel"

    if {[llength $arg] == "1"} {set arg "1 $arg"}

    if {[dbNumRecs $file] == 0} {
	tellnick $errortype $who "There are no quotes."
	return
    }

    set num [lindex [split $arg] 0]
    regsub -all {[0-9]} $num "" num_nonum
    if {$num_nonum == "" && $num != ""} {
	set arg [join [lrange [split $arg] 1 end]]
    } else {
	set num 1
    }
    debug "Num: $num"
    
    if {[llength $arg] == "0"} {
	debug "RQUOTE: Not enough args"
	tellnick $errortype $who "Rquote: Usage: !rquote ?num? searchstring"
	return
    }
    
    if {$arg != ""} {
	debug "RQUOTE: Search would be: $arg"

	set listargs [split $arg ""]
	set numwild 0
	foreach argitem $listargs {
	    if {[regexp {\*} $argitem] == "1"} {incr numwild}
	    if {[regexp {\?} $argitem] == "1"} {incr numwild}
	}
	if {$numwild >= "4"} {
	    debug "QUOTE: Too many wildcards."
	    tellnick $errortype $nick "Match: Too many wildcards."
	    return
	}
	
	if {[string trim $arg "*"] == ""} {
	    debug "RQUOTE: User requested *"
	    set ms {[cached]}
	    if {[dbNumRecs $file] == "1"} {set s ""} else {set s "s"}
	    set displaymax [get_config ${a}_$channel displaymax]
	    if {$num > $displaymax} {
		set num $displaymax
	    }

	    if {$num == 1} {set ss ""} else {set ss "s"}

	    tellnick $type $channel "$ms Rquote: Showing ${num} random quote$ss of\
		    [dbNumRecs $file] quote$s matching \"${arg}\"."
	    set found ""
	    for {set x 0} {$x < $num} {incr x} {
		set quote [random_quote $file]
		if {[lsearch -exact $found $quote] == "-1"} {
		    lappend found $quote
		} else {
		    while {[lsearch -exact $found $quote] != "-1"} {
			set quote [random_quote $file]
		    }
		    lappend found $quote
		}
	    }

	    display_quotes $file $type $channel $found
            random_quote_reset $channel
	    return
	}

	start_bench
	set found [dbMatchingRecords $file quote *${arg}*]
	set ms [stop_bench]

	set numfound [llength $found]
	debug "RQUOTE: found: $found"

	if {$num > $numfound} {
	    set num $numfound
	}

	if {$found == ""} {
	    debug "RQUOTE: No quotes found matching search"
	    tellnick $errortype $who "Rquote: No quotes matching \"${arg}\""
	    return
	} else {
	    if {$numfound == "1"} {set s ""} else {set s "s"}
	    if {$num == 1} {set ss ""} else {set ss "s"}

	    debug "RQUOTE: Found $numfound quote$s matching $arg." 
	    
	    if {$num != ""} {
		if {$num >= $displaymax} {
		    set num $displaymax
		}
		debug "RQUOTE: Num: $num"
		
		set ranlist ""
		for {set x 0} {$x != $num} {incr x} {
		    set quote [lindex $found [random $numfound]]
		    if {[lsearch -exact $ranlist $quote] == "-1"} {
			lappend ranlist $quote
		    } else {
			while {[lsearch -exact $ranlist $quote] != "-1"} {
			    set quote [lindex $found [random $numfound]]
			}
			lappend ranlist $quote
		    }
		}
		debug "RQUOTE: List of ranquotes: $ranlist"

		tellnick $type $channel "$ms Rquote: Showing ${num} random quote$ss of\
			${numfound} quote$s matching \"${arg}\"."
		display_quotes $file $type $channel $ranlist
                random_quote_reset $channel
		return
	    }
	    return
	}
	return
    }
    return
}

proc pub_next {nick uhost handle channel arg} {
    variable a
    set channel [string tolower $channel]
    if {[get_config ${a}_$channel enable] == 0} {return}    
    if {[catch {datafile_load_forchannel $a $channel} error]} {putlog $error; return}
    if {![check_access $nick $channel $handle\
            [get_config ${a}_$channel !next]\
            [get_config ${a}_$channel msg]]} {
        return
    }

    if {[check_quotedelay $nick $handle $channel]} {return}

    return
}

proc pub_quotebucket {nick uhost handle channel {arg ""}} {
    variable a
    set channel [string tolower $channel]
    if {[get_config ${a}_$channel enable] == 0} {return}
    if {[catch {datafile_load_forchannel $a $channel} error]} {putlog $error; return}
    if {![check_access $nick $channel $handle\
            [get_config ${a}_$channel !quotebucket]\
            [get_config ${a}_$channel msg]]} {
        return
    }

    set file       [get_config ${a}_$channel datafile]

    set type "pub"
    set errortype "notice"
    set who $nick
    
    set sherrors   [get_config ${a}_$channel sherrors]
    set displaymax [get_config ${a}_$channel displaymax]

    set args $arg
    set arg1 [lindex $arg 0]
    set arg2 [lindex $arg 1]

    debug "---------------"
    debug "QUOTEBUCKET: ARG1: $arg1"
    debug "QUOTEBUCKET: ARG2: $arg2"
    
    set numqbkt [dbNumRecs $file.QuoteBucket]
    
    if {$arg == ""} {
	tellnick $errortype $nick "Usage: !quotebucket <options>\
		For help type !quotebucket -help"
	return
    }

    switch -- $arg1 {
	-list {
	    if {$numqbkt == 0} {
		tellnick $errortype $who "QuoteBucket: There are no quotes in bucket."
		return
	    }

	    if {$arg2 == ""} {
		set firstnumber 0
	    } else {
		set firstnumber $arg2
	    }		
	    
	    set secondnumber [expr $firstnumber + $displaymax -1]
	    set found ""
	    for {set x 0} {$x < $numqbkt} {incr x} {
		lappend found $x
	    }
	    
	    set displayone [expr $firstnumber +1]
	    set displaytwo [expr $secondnumber +1]
	    
	    debug "QUOTEBUCKET: $channel: Number of quotes in bucket: $numqbkt"
	    debug "QUOTEBUCKET: $channel: First Num: $firstnumber"
	    debug "QUOTEBUCKET: $channel: Secnd Num: $secondnumber"
	    debug "QUOTEBUCKET: $channel: DisplayingOne: $displayone"
	    debug "QUOTEBUCKET: $channel: DisplayingTwo: $displaytwo"
	    debug "QUOTEBUCKET: $channel: List of indexes: $found"
	    
	    if {$firstnumber != "0"} {
		set found [lrange $found $displayone end]
	    }
	    
	    set numleft [llength $found]
	    set message "Displaying QuoteBucket contents."
	    
	    if {$numleft <= $displaymax} { 
		tellnick $type $channel "$message"
	    } else {
		tellnick $type $channel "$message To display the next 5, type !quotebucket -list $displaytwo"
	    }

	    display_quotebucket $file $type $channel $found
	    return  
	}

	-restore {
	    if {$numqbkt == 0} {
		tellnick $errortype $who "QuoteBucket: There are no quotes in bucket."
		return
	    }

	    if {$arg2 == ""} {
		tellnick $errortype $nick "QuoteBucket: Usage: !quotebucket -restore <num>"
		return
	    }
	    
	    regsub -all {[0-9]} $arg2 "" nonum
	    if {$nonum != ""} {
		tellnick $errortype $nick "QuoteBucket: Argument must be a number."
		return
	    }
	    
	    set found [dbBinarySearch $file.QuoteBucket bucket_id $arg2]
            if {$found == "-1"} {
                set id [dbField $file.QuoteBucket bucket_id $arg2 absolute]
                if {$id != ""} {
                    set found [dbBinarySearch $file.QuoteBucket bucket_id $id]
                } else {
                    tellnick $errortype $nick "QuoteBucket: Quote #$arg2 does not exist."
                    return
                }
            }

            set quote [dbField $file.QuoteBucket quote $found]

	    if {[get_config ${a}_$channel nocolors]} {set quote [ridcolor $quote]}

            set add ""
	    lappend add $quote
	    lappend add [dbField $file.QuoteBucket author $found]
	    lappend add [dbField $file.QuoteBucket date   $found]
	    lappend add [dbField $file.QuoteBucket locked $found]
	    dbInsert $file $add
	    dbDelete $file.QuoteBucket $found

	    datafile_save $a $file
	    tellnick $errortype $who "QuoteBucket: Restored quote."
	    return
	}

	-purge {
	    if {![check_access $nick $channel $handle\
		    [get_config ${a}_$channel "!quotebucket -purge"]\
		    [get_config ${a}_$channel msg]]} {
		return
	    }
	    
	    if {$numqbkt == 0} {
		tellnick $errortype $who "QuoteBucket: There are no quotes in bucket."
		return
	    }

	    dbCreate $file.QuoteBucket $file.QuoteBucket {quote author date locked bucket_time bucket_id whodel}
            dbSort $file.QuoteBucket quote
	    dbWrite $file.QuoteBucket
	    tellnick $errortype $who "QuoteBucket: Purged successfully."
	    return
	}

	-info {
	    if {$arg2 == ""} {
		tellnick $errortype $nick "Usage: !quotebucket -info <num>"
		return
	    }
	    regsub -all {[0-9]} $arg2 "" nonum2
	    if {$nonum2 != ""} {
		tellnick $errortype $nick "Info: Argument must be a number."
		return
	    }
	    
	    set numqbkt [dbNumRecs quotebucket]
	    if {$arg2 >= $numqbkt} {
		tellnick $errortype $nick "Info: There is no quote in the bucket of\
			number: $arg2"
		return
	    }
	    
	    set quote  [dbField $file.QuoteBucket quote  $arg2]
	    set author [dbField $file.QuoteBucket author $arg2]
	    set date   [dbField $file.QuoteBucket date   $arg2]
	    set whodel [dbField $file.QuoteBucket whodel $arg2]
	    
	    tellnick pub $channel "$quote"
	    tellnick pub $channel "Trashed by: $whodel"
	    tellnick pub $channel "Author: $author"
	    tellnick pub $channel "Trashed: [lrange [since $date] 0 1] ago ([clock format $date])"
	    return
	}
	-help {
	    tellnick $errortype $who "!QuoteBucket Arguments:"
	    tellnick $errortype $who "-help          - Display help (this text)"
	    tellnick $errortype $who "-list          - Display the contents of the quotebucket"
	    tellnick $errortype $who "-info    <num> - Display info of a trashed quote"
	    tellnick $errortype $who "-restore <num> - Restore quote <num> from the quotebucket"
	    tellnick $errortype $who "KobazQuote.tcl written and directed by Kobaz"
	    return
	}
    }
    tellnick notice $nick "Invalid argument and/or usage.  !quotebucket -help for help."
    return
}

proc pub_match {nick uhost handle channel arg} {
    variable a
    set channel [string tolower $channel]
    if {[get_config ${a}_$channel enable] == 0} {return}    
    if {[catch {datafile_load_forchannel $a $channel} error]} {putlog $error; return}
    if {![check_access $nick $channel $handle\
            [get_config ${a}_$channel !match]\
            [get_config ${a}_$channel msg]]} {
        return
    }

    set file    [get_config ${a}_$channel datafile]

    set sherrors [get_config ${a}_$channel sherrors]
    
    set type "pub"
    set errortype "notice"
    set who $nick
    set args $arg

    if {[check_quotedelay $nick $handle $channel]} {return}

    debug "---------------"
    debug "MATCH: ARGS: $arg"
    
    if {[dbNumRecs $file] == 0} {
	tellnick $errortype $who "There are no quotes."
	return
    }
    
    if {$arg == ""} {
	tellnick $errortype $nick "Usage: !match <string>"
	return
    }

    if {$arg != ""} {
	set listargs [split $arg ""]
	set numwild 0
	foreach argitem $listargs {
	    if {[regexp {\*} $argitem] == "1"} {incr numwild}
	    if {[regexp {\?} $argitem] == "1"} {incr numwild}
	}
	if {$numwild >= "5"} {
	    debug "QUOTE: Too many wildcards."
	    tellnick $errortype $nick "Match: Too many wildcards."
	    return
	}

	if {[string trim $arg "*"] == ""} {
	    debug "MATCH: User requested *"
	    set ms {[cached]}
	    tellnick $type $channel "$ms [dbNumRecs $file] quotes matching \"$arg\". 100%"
	    return
	}

	######
	######
	######

	if {1 == 0} {
	    set cached 0
	    debug "QUOTE: Searches Cached: $KobazQuote::cachelist"
	    if {[lsearch $KobazQuote::cachelist $arg] != "-1"} {
		debug "MATCH: Search was cached (phew)"
		set found $KobazQuote::cache($arg)
		set ms {[cached]}
		set cached 1
	    }
	    
	    if {$cached == 0} {
		debug "MATCH: Search not cached, cacheing"
		start_bench
		set found [dbMatchingRecords quotes quote *${arg}*]
		set ms [stop_bench]
		
		set KobazQuote::cache($arg)   $found
		lappend KobazQuote::cachelist $arg
		set maxcache [get_config $a maxcache]
		if {[llength $KobazQuote::cachelist] > $maxcache} {
		    debug "MATCH: Cache too large, removing one entry"
		    set KobazQuote::cachefirst [lindex $KobazQuote::cachelist 0]
		    unset KobazQuote::cache($KobazQuote::cachefirst)
		    set KobazQuote::cachelist [lreplace $KobazQuote::cachelist 0 0]
		    debug "MATCH: Searches Cached: $KobazQuote::cachelist"
		}
	    }
	}								

	######
	######
	######	

	regsub -all {\\|\[|\]} $arg {\\&} arg2

	start_bench
	set found [dbMatchingRecords $file quote *${arg2}*]
	set ms [stop_bench]
	
	set numfound [llength $found]
	if {$numfound < 2} {set s ""} else {set s "s"}
	if {$numfound == 1} {set s ""}
	if {$numfound == 0} {set s "s"}

	set numfound [llength $found]
	
	if {$numfound == 0} {
	    tellnick $errortype $who "Match: No quotes matching \"${arg}\"."
	    return
	}
	
	set percent [expr double($numfound)/[dbNumRecs $file] * 100]
	set percent [format %.2f $percent]
	debug "MATCH: Number of matches: $numfound"
	tellnick $type $channel "$ms $numfound quote$s matching \"${arg}\". ${percent}%"
    }
}

proc pub_info {nick uhost handle channel arg} {
    variable a
    set channel [string tolower $channel]
    if {[get_config ${a}_$channel enable] == 0} {return}    
    if {[catch {datafile_load_forchannel $a $channel} error]} {putlog $error; return}
    if {![check_access $nick $channel $handle\
            [get_config ${a}_$channel !info]\
            [get_config ${a}_$channel msg]]} {
        return
    }

    variable data

    set file       [get_config ${a}_$channel datafile]

    set sherrors [get_config ${a}_$channel sherrors]

    set type "pub"
    set errortype "notice"
    set who $nick

    if {[check_quotedelay $nick $handle $channel]} {return}
    
    debug "---------------"
    debug "INFO: ARGS: $arg"
    
    if {[llength $arg] == "0"} {
	debug "INFO: Not enough args"
	tellnick $errortype $who "Usage: !info quotenum"
	return
    }

    set firstarg [lindex $arg 0]

    switch -- $firstarg {
	-last {
	    debug "QUOTE: $channel: User requested last randomly displayed quote."
	    if {$data(LastRanQuote,$channel) == ""} {
		tellnick $errortype $nick "Info: No random quote said yet."
		return
	    }
	    set LastRanQuoteNum [dbBinarySearch $file quote $data(LastRanQuote,$channel)]
	    if {$LastRanQuoteNum == "-1"} {
		tellnick $errortype $nick "Info: Last random quote was deleted."
		return
	    }

            set quote  $LastRanQuoteNum
            set author [dbField $file author $quote]
            set locked [dbField $file locked $quote]
            set quotedate [dbField $file date $quote]
            if {$locked == "1"} {set author "Entry is Locked"}

	    display_quotes $file $type $channel $quote
            tellnick $type $channel "Author: $author"
            tellnick $type $channel "Added : [lrange [since $quotedate] 0 1] ago\
                    ([clock format $quotedate])"
            
            random_quote_reset $channel
	    return
	}
	
	-lastadded {
	    debug "QUOTE: $channel: User requested last added quote."
	    if {$data(LastAddedQuote,$channel) == ""} {
		tellnick $errortype $nick "Info: Last Added Quote was not recorded."
		return
	    }   
	    set LastAddedQuoteNum [dbBinarySearch $file quote $data(LastAddedQuote,$channel)]
	    if {$LastAddedQuoteNum == "-1"} {
		tellnick $errortype $nick "Info: Last Added Quote was Deleted."
		return
	    }

            set quote $LastAddedQuoteNum

            set author [dbField $file author $quote]
            set locked [dbField $file locked $quote]
            set quotedate [dbField $file date $quote]
            if {$locked == "1"} {set author "Entry is Locked"}

	    display_quotes $file $type $channel $quote
            tellnick $type $channel "Author: $author"
            tellnick $type $channel "Added : [lrange [since $quotedate] 0 1] ago\
                    ([clock format $quotedate])"


            random_quote_reset $channel
	    return
	}

	-help {
	    tellnick $errortype $who "!info Arguments:"
	    tellnick $errortype $who "-help      - Display help (this text)"
	    tellnick $errortype $who "-last      - Display the last randomly displayed quote"
	    tellnick $errortype $who "-lastadded - Display the last added quote"
	    tellnick $errortype $who "KobazQuote.tcl written and directed by Kobaz"
	    return
	}
    }
    
    if {$arg != ""} {
	debug "INFO: Quote would be: $arg"
	
	regsub -all {[0-9]} $arg "" nonumarg
	if {$nonumarg != ""} {
	    tellnick $errortype $nick "Info: Argument must be a number."
	    return
	}
	
        set quote [dbField $file quote $arg]
        if {$quote == ""} {
	    tellnick $errortype $nick "Info: There is no quote number: $arg"
	    return
	}
	
	set author [dbField $file author $arg]
	set locked [dbField $file locked $arg]
	set quotedate [dbField $file date $arg]
	
	if {$locked == "1"} {set author "Entry is Locked"}
	
	display_quotes $file $type $channel $arg nonum
	tellnick $type $channel "Author: $author"
	tellnick $type $channel "Added : [lrange [since $quotedate] 0 1] ago ([clock format $quotedate])"
    }
    return
}

proc pub_lock {nick uhost handle channel arg} {
    variable a
    set channel [string tolower $channel]
    if {[get_config ${a}_$channel enable] == 0} {return}    
    if {[catch {datafile_load_forchannel $a $channel} error]} {putlog $error; return}
    if {![check_access $nick $channel $handle\
            [get_config ${a}_$channel !quotelock]\
            [get_config ${a}_$channel msg]]} {
        return
    }

    set file       [get_config ${a}_$channel datafile]
    
    set errortype notice
    set who $nick
    set sherrors [get_config ${a}_$channel sherrors]
    
    debug "LOCK: $channel: ARGS: $arg"
    
    set yesno [lindex $arg 0]
    set quotenum [lindex $arg 1]
    
    if {[llength $arg] < 2} {
	debug "LOCK: $channel: Not enough args"
	tellnick $errortype $who "Usage: !quotelock on/off quotenum"
	return
    }
    
    if {$yesno == "on"} {set statuz 1}
    if {$yesno == "off"} {set statuz 0}
    
    set status [dbField $file locked $quotenum]
    
    if {$arg != ""} {
	if {$yesno == "on"} {
	    if {$status == $statuz} {
		tellnick $errortype $who "Lockquote: Quote $arg is already locked."
		return
	    }
	    debug "LOCK: $channel: Locking quote: $quotenum"
	    tellnick $errortype $who "Lockquote: Quote $arg is now locked."
	    
	    set add ""
	    lappend add [dbField $file quote  $quotenum]
	    lappend add [dbField $file author $quotenum]
	    lappend add [dbField $file date   $quotenum]
	    lappend add "1"

	    dbDelete $file $quotenum
	    dbInsert $file $add
	    datafile_save $a $file	
    
	    return
	}
	
	if {$yesno == "off"} {
	    if {$status == $statuz} {
		tellnick $errortype $who "Lockquote: Quote $arg is already not locked."
		return
	    }
	    
	    debug "LOCK: $channel: Unlocking quote: $quotenum"
	    tellnick $errortype $who "Lockquote: Quote $quotenum is now unlocked."
	    
	    set add ""
	    lappend add [dbField $file quote  $quotenum]
	    lappend add [dbField $file author $quotenum]
	    lappend add [dbField $file locked $quotenum]
	    lappend add "0"
	    
	    dbDelete $file $quotenum
	    dbInsert $file $add
	    datafile_save $a $file
	    
	    return
	}
	
	
    }
    return
}

##########################################
########### End of Major Code ############
##########################################

init

}

