###############################################################################
## TCL SQL Library 2.2.4
## KobazSQL-Lib.tcl
## First Public Release
## 
## AUTHOR: Kobaz Mi'Daire <krobaz@yahoo.com>
## 
## REQUIRES: Eggdrop 1.3.x AND tcl 8.1
##           my-tcl sql library for tcl
##
## PURPOSE:  Provide a simpler interface to the SQL library
##
###############################################################################

source scripts/kobazscripts/KobazDep.tcl
kbrequiretcl 8.1

namespace eval KobazSQL {

############
# Settings #
############

# Location of the mysql-tcl library
set sql_lib_file "./mytcl.so"

#################
# Settings -End #
#################

############################################
############################################
## Modify contents below at your own risk ##
############################################
############################################

namespace export sql_connect
namespace export sql_disconnect
namespace export sql_isconnected
namespace export sql_selectdb
namespace export sql_exec
namespace export sql_query
namespace export sql_endquery
namespace export sql_create_db
namespace export sql_create_table
namespace export sql_drop_table
namespace export sql_fixforsearch
namespace export sql_fixforadd
namespace export sql_unfixforadd
namespace export sql_fetchrow
namespace export sql_fetchallrows
namespace export sql_countrows
namespace export sql_lastrec
namespace export sql_delete
namespace export sql_modify
namespace export sql_getrec
namespace export sql_getfield
namespace export sql_getabsoluterec
namespace export sql_getabsolutefield
namespace export sql_numrecs
namespace export sql_reindex
namespace export sql_search

set a KobazSQL-Lib

if {![info exists firstload]} {
    set conn ""
    set firstload "1"
}

proc init {} {
    variable a
    variable mytcl_have
    variable sql_lib_file

    set mytcl_have 1
    if {![file exists $sql_lib_file]} {
        putlog "Cannot load tcl-sql library: $sql_lib_file, Make sure this file is in your eggdrop directory!"
        putlog "You will not be able to use mysql features without this file. This file is not required."
        set mytcl_have 0
    }
    if {([catch {load $sql_lib_file} error]) && ($mytcl_have)} {
        putlog "Cannot load tcl-sql library: $sql_lib_file, $error"
        putlog "You will not be able to use mysql features without this file working. This file is not required."
        set mytcl_have 0
    }
    catch sql result
    if {(![string match "*Usage:*" $result]) && ($mytcl_have)} {
        putlog "Invalid tcl-sql library file: $sql_lib_file"
        putlog "You will not be able to use mysql features without this file working. This file is not required."
        set mytcl_have 0
    }
    unset result

    putlog "$a.tcl by Kobaz loaded."
}

proc sql_connect {host user pass} {
    variable conn

    set conn [sql connect $host $user $pass]

    return $conn
}

proc sql_disconnect {} {
    variable conn

    return [sql disconnect $conn]
}

proc sql_isconnected {} {
    variable conn

    if {[catch {sql isconnected $conn}]} {
        return 0
    } else {
        return 1
    }
}

proc sql_selectdb {db} {
    variable conn
    variable selected_db
    
    set selected_db $db

    return [sql selectdb $conn $db]
}

proc sql_exec {cmd} {
    variable conn

    catch {sql exec $conn $cmd} error

    if {$error == ""} {set error "0"}

    return $error
}

proc sql_query {cmd} {
    variable conn

    set data ""
    sql query $conn $cmd
    set data [sql_fetchallrows]
    sql_endquery

    return $data
}

proc sql_endquery {} {
    variable conn

    return [sql endquery $conn]
}

proc sql_create_db {db} {
    return [sql_exec "CREATE DATABASE $db"]
}

proc sql_create_table {table cols} {
    return [sql_exec "CREATE TABLE $table $cols"]
}

proc sql_drop_table {table} {
    return [sql_exec "DROP TABLE $table"]
}

proc sql_fixforsearch {string} {
#    regsub -all {\\} $string {\\&} string
    regsub -all {'} $string {\\&} string
    regsub -all {%} $string {\\&} string
    regsub -all {\*} $string "%" string

    return $string
}

proc sql_fixforadd {string} {
    regsub -all {\\} $string {\\&} string
    regsub -all {'} $string {\\&} string

#    regsub -all {\{} $string {\\&} string
#    regsub -all {\}} $string {\\&} string

    return $string
}

proc sql_unfixforadd {string} {
#    regsub -all {\\'} $string {'} string

    return $string
}

proc sql_fetchrow {} {
    variable conn

    set row [sql fetchrow $conn]

#    if {([string index $row 0] == "\{") && ([string index $row end] == "\}") && ([string match "*\*" $row])} {
#        set row "[string range $row 1 end-1]"
#    }

    return [lindex $row 0]
}

proc sql_fetchallrows {} {
    set rows ""

    while {[set row [sql_fetchrow]] != ""} {
        lappend rows $row
    }

    return "$rows"
}

proc sql_countrows {search} {
    variable conn
 
    set search [sql_fixforsearch $search]
    return [sql_query "SELECT count(quote) FROM quotes WHERE quote LIKE '$search'"]
}

proc sql_lastrec {table} {
    return [sql_query "SELECT recnum FROM $table LIMIT [expr [sql_numrecs $table] - 1],1"]
}

proc sql_delete {table recnum} {
    set found [sql_query "SELECT recnum FROM $table WHERE recnum = $recnum"]
    if {$found == ""} {return 1}
    set result [sql_exec "DELETE FROM $table WHERE recnum = $recnum"]
    return $result
}

proc sql_getrec {table recnum} {
    return [sql_query "SELECT * FROM $table WHERE recnum = $recnum"]
}

proc sql_getfield {table field recnum} {
    return [lindex [sql_query "SELECT $field FROM $table WHERE recnum = $recnum"] 0]
}

proc sql_getabsoluterec {table num} {
    return [lindex [sql_query "SELECT * FROM $table LIMIT $num,1"] 0]
}

proc sql_getabsolutefield {table field num} {
    return [sql_query "SELECT $field FROM $table LIMIT $num,1"]
}

proc sql_numrecs {table} {
    return [sql_query "SELECT count(*) FROM $table"]
}

proc sql_modify {table fieldname recnum data} {
    set data [sql_fixforadd $data]
    sql_exec "UPDATE $table SET $fieldname='$data' WHERE recnum = $recnum"
    return
}

proc sql_reindex {table} {
    set numrecs [sql_numrecs $table]

    for {set x 0} {$x < $numrecs} {incr x} {
        set query [sql_getabsoluterec $x]
        set num [lindex $query 0]
        if {$num != $x} {sql_exec "UPDATE $table SET recnum='$x' WHERE recnum = $num"}
    }
    return
}

proc sql_search {table field search} {
    set search [sql_fixforsearch $search]
    return [sql_query "SELECT recnum FROM $table WHERE $field LIKE '$search' ORDER BY $field"]
}

init

}
