###############################################################################
## Triva Game 2.2.4
## KobazTrivia.tcl
## 
## AUTHOR: Kobaz Mi'Daire <krobaz@yahoo.com>
## 
## REQUIRES: Eggdrop 1.3.x AND tcl 8.1
##           KobazDatabase.tcl
##           KobazConfig.tcl
##           KobazTools.tcl
##           KobazHelp.tcl
##
## FEATURES: Trivia Game
##           Simaltanious Trivia games in different channels
##           Completely Customizeable via dcc.
## 
## COMMANDS: Arguments in "< >" are mandatory.
##           Arguments in "? ?" are optional.  
##           Arguments in "[ ]" are a toggle, and mandatory.
##           Arguments in "( )" must be used together
## 
## !trivia               - Starts a trivia game in the current channel
## !killtrivia           - Stop trivia game in current channel
## !addtrivia            - Add a trivia question 
##
###############################################################################

###############################################################################
# All Configuration is done via the bot now: the .kbconfig KobazTrivia command
# If trivia commands do not work in a specific channel, remember to use
# .kbconfig KobazTrivia to set enable to 1 for that channel
###############################################################################

############################################
############################################
## Modify contents below at your own risk ##
############################################
############################################

source scripts/kobazscripts/KobazDep.tcl
kbrequiretcl 8.1

namespace eval KobazTrivia {
set a KobazTrivia

####
# Load required libraries

kbrequire DatabaseAPI
kbrequire Config
kbrequire Tools
kbrequire Help

# end
####

namespace import -force ::KobazTools::*
namespace import -force ::KobazHelp::*
namespace import -force ::KobazConfig::*
namespace import -force ::KobazDBAPI::*

############################
# Settings - Do NOT Modify #
############################
proc add_configs {} {
variable a
add_config $a CREATE . . .
add_config $a "tog global" debug "0" {Enable debug? You probebly really don't need to turn this on}
add_config $a "text global" filterout {"the" "a"} {Words to filter out of guess-answer comparison}
add_config $a "tog" enable "0" {Enable trivia commands on this channel?}
add_config $a "tog" gametype "1" {Game type, 0=points 1=rounds}
add_config $a "num_no0" points "10" {If gametype is points, then points needed to win}
add_config $a "num_no0" rounds "10" {If gametype is rounds, then game ends after this number of rounds}
add_config $a "num_no0" idlekilltime "5" {Time in minutes to kill game if no one answers a question}
add_config $a "num_no0" roundtime "60" {Time in seconds that rounds last}
add_config $a "tog" showanswer "0" {Show answer for questions that were not correctly answered}
add_config $a "text" datafile "KobazTrivia.db" {Data file to use for this channel}
add_config $a flag whoplay     "-"  "Flags for who can answer questions in the game"
add_config $a flag !trivia     "o"  "Flags to access !trivia"
add_config $a flag !killtrivia "o"  "Flags to access !killtrivia"
add_config $a flag !addtrivia  "o"  "Flags to access !addtrivia"
add_config $a text_star msg "No Access." {Message to give person if denied access to a command. If value is * then no message will be given.}
}
#################
# Settings -End #
#################

#################
# Help          #
#################
proc add_helps {} {
variable a
addhelp $a "!trivia"       "Start trivia game"
addhelp $a "!killtrivia"   "Stop trivia game"
addhelp $a "!addtrivia"    "Add a trivia question"
}
#################
# Help -End     #
#################

if {![info exists firstload]} {
    add_helps
    add_configs
    if {[catch {load_config $a}]} {
        setup_globals $a
    }
    set_config $a fields "subject question answer"
    set_config $a dojoin 1
    set firstload "1"
}

proc init {} {
    variable add
    variable a

    bind pub  - !trivia        KobazTrivia::pub_starttrivia
    bind pub  - !killtrivia    KobazTrivia::pub_killtrivia
    bind msg  - !addtrivia     KobazTrivia::msg_add
    bind pub  - !addtrivia     KobazTrivia::pub_add

    putlog "${a}.tcl: Loading Triviafiles(s)..."
    datafile_loadall $a
    
    putlog "${a}.tcl by Kobaz Loaded."
    return
}

proc error {args} {
    putlog "TRIVIA=:=ERROR $args"
}

proc debug {args} {
    variable a
    set debug [get_config $a debug]

    if {$debug == "1"} {
	putlog "TRIVIA:DEBUG $args"
    }
}

proc pub_starttrivia {nick uhost handle channel arg} {
    variable a
    variable cur

    set channel [string tolower $channel]
    if {[catch {datafile_load_forchannel $a $channel} error]} {putlog $error; return}

    if {[get_config ${a}_$channel enable] == 0} {return}
    if {![check_access $nick $channel $handle\
	    [get_config ${a}_$channel !trivia]\
	    [get_config ${a}_$channel msg]]} {
	return
    }

    set errortype notice

    set datafile [get_config ${a}_$channel datafile]
    if {[dbNumRecs $datafile] < "1"} {
        KobazTools::tellnick $errortype $nick "There are no trivia questions."
        return
    }

    set rounds [get_config ${a}_$channel rounds]
    set points [get_config ${a}_$channel points]
    set numrecs [dbNumRecs $datafile]

    if {($numrecs < $rounds) || ($numrecs < $points)} {
        set gametype [get_config ${a}_$channel gametype]
        switch -- $gametype {
            0 {
                KobazTools::tellnick $errortype $nick "There are not enough questions to start a game. There needs to be at least $points"
            }
            1 {
                KobazTools::tellnick $errortype $nick "There are not enough questions to start a game. There needs to be at least $rounds"
            }
        }
        return
    }
   
    if {[info exists cur($channel,started)] && $cur($channel,started) == "1"} {
	KobazTools::tellnick $errortype $nick "Trivia game already in progress."
	return
    }

    debug "START: Starting Trivia Game"

    if {[catch {KobazDB::dbCreate kbtrivia_scores_$channel "" {nick score} nosave} error]} {
        return -code error "error creating scores db: $error"
    }

    KobazDB::dbSort kbtrivia_scores_$channel nick

    set type pub

    set gametype [get_config ${a}_$channel gametype]
    if {$gametype == "1"} {
	KobazTools::tellnick $type $channel "Starting Trivia Game. Game ends after $rounds rounds"
    } else {
	KobazTools::tellnick $type $channel "Starting Trivia Game. First one to get $points points, wins!"
    }

    set cur($channel,started)     "1"
    set cur($channel,round)       "1"
    set cur($channel,aqs)         ""

    bind pubm - "$channel =*"  "KobazTrivia::pub_guess"
    bind sign - "$channel *"   "KobazTrivia::remove_score_quit"
    bind part - "$channel *"   "KobazTrivia::remove_score_quit"
    bind kick - "$channel *"   "KobazTrivia::remove_score_kick"
    bind nick - "$channel *"   "KobazTrivia::change_score_nick"

    timer [get_config ${a}_$channel idlekilltime] "KobazTrivia::idletimer $channel"
    trivia $channel 0

    return
}

proc pub_killtrivia {nick uhost handle channel arg} {
    variable a
    variable cur

    set channel [string tolower $channel]

    if {[get_config ${a}_$channel enable] == 0} {return}
    if {![check_access $nick $channel $handle\
	    [get_config ${a}_$channel !killtrivia]\
	    [get_config ${a}_$channel msg]]} {
	return
    }

    set who $nick
    set errortype notice

    if {![info exists cur($channel,started)] || $cur($channel,started) == "0"} {
	KobazTools::tellnick $errortype $nick "No trivia game in progress"
	return
    }

    kill_game $channel
    KobazTools::tellnick pub $channel "Trivia Game ended."
    return
}

proc kill_round {channel} {
    variable cur

    set cur($channel,inround) "0"

    foreach timernum "1 2 3" {
	set timer [utimerexists "KobazTrivia::trivia $channel $timernum"]
	if {$timer != ""} {
	    killutimer $timer
	}
    }

    return
}

proc unbinds {channel} {
    catch {unbind pubm - "$channel = *"  "KobazTrivia::pub_guess"}
    catch {unbind sign - "$channel *"    "KobazTrivia::remove_score_quit"}
    catch {unbind part - "$channel *"    "KobazTrivia::remove_score_quit"}
    catch {unbind kick - "$channel *"    "KobazTrivia::remove_score_kick"}
    catch {unbind nick - "$channel *"    "KobazTrivia::change_score_nick"}
    
    return
}

proc kill_game {channel} {
    variable cur

    set channel [string tolower $channel]

    kill_idletimer $channel
    kill_round     $channel

    set killvars "
	cur($channel,started)
	cur($channel,round)
	cur($channel,inround)
	cur($channel,q)
	cur($channel,qf)
	cur($channel,af)
	cur($channel,aqs)
	cur($channel,subject)
    "
    
    foreach killvar [subst $killvars] {
	catch {unset $killvar}
    }

    KobazDB::dbClose kbtrivia_scores_$channel

    utimer 10 "KobazTrivia::unbinds $channel"

    debug "KILLTRIVIA: Ended game on $channel"
    return
}

proc set_score {nick channel score {incr ""}} {
    set channel [string tolower $channel]

    if {[findscore $channel $nick _score index]} {
        if {$incr != ""} {
            set score [incr _score $incr]
        }
        KobazDB::dbModify kbtrivia_scores_$channel score $index $score
	debug "SETSCORE: Nick: $nick -- score changed to $score"
        return 1
    } else {
        KobazDB::dbInsert kbtrivia_scores_$channel "$nick $score"
	debug "SETSCORE: Nick: $nick -- score set to $score"
        return 1
    }

    return 0
}

proc get_highestscore {channel} {
    set channel [string tolower $channel]
    set last [expr [KobazDB::dbNumRecs kbtrivia_scores_$channel] - 1]

    KobazDB::dbSort kbtrivia_scores_$channel score -decreasing
    set highestscore [KobazDB::dbField kbtrivia_scores_$channel score $last]
    KobazDB::dbSort kbtrivia_scores_$channel nick
    
    return $highestscore
}

proc remove_score {nick channel} {
    set channel [string tolower $channel]

    if {[findscore $channel $nick score index]} {
	debug "REMOVESCORE: Nick: $nick -- quit/parted/got kicked from $channel, removeing score"
        KobazDB::dbDelete kbtrivia_scores_$channel $index
        return 1
    }

    return 0
}

proc findscore {channel nick score index} {
    upvar $score myscore
    upvar $index myindex

    set index [KobazDB::dbBinarySearch kbtrivia_scores_$channel nick $nick]
    if {$index != -1} {
        set myscore [KobazDB::dbField kbtrivia_scores_$channel score $index]
        set myindex $index
        return 1
    }
    
    return 0
}


proc change_nick {channel oldnick newnick} {
    global botnick
    if {$oldnick == $botnick} {
	return
    }

    if {[findscore $channel $oldnick score index]} {
	debug "CHANGESCORE: $oldnick changed nick to $newnick on ${channel}, Changing name to match score."
        KobazDB::dbModify kbtrivia_scores_$channel nick $index $score
        return 1
    }

    return 0
}

proc remove_score_quit {nick uhost handel channel {reason ""} } {
    remove_score $nick $channel
    return
}

proc remove_score_kick {nick uhost handel channel kickednick reason} {
    remove_score $kickednick $channel
    return
}

proc change_score_nick {nick uhost handle channel newnick} {
    change_nick $channel $nick $newnick
    return
}

proc kill_idletimer {channel} {
    set idletimer [timerexists "KobazTrivia::idletimer $channel"]
    if {$idletimer != ""} {
        killtimer $idletimer
    }

    return
}

proc idletimer {channel} {
    kill_idletimer $channel

    KobazTools::tellnick pub $channel "Trivia Game has ended due to lack of\
	    participation. Final scores:"

    show_scores $channel win    

    return
}

proc pub_guess {nick uhost handle channel arg} {
    variable a
    variable cur

    set channel [string tolower $channel]
    if {[catch {datafile_load_forchannel $a $channel} error]} {putlog $error; return}

    if {[get_config ${a}_$channel enable] == 0} {return}
    if {![check_access $nick $channel $handle\
	    [get_config ${a}_$channel whoplay]\
	    [get_config ${a}_$channel msg]]} {
	return
    }

    if {![info exists cur($channel,started)] || $cur($channel,started) != "1"} {return}
    if {![info exists cur($channel,inround)] || $cur($channel,inround) != "1"} {return}

    kill_idletimer $channel

    regexp {=(.*)} $arg junk guess

    set answer $cur($channel,af)

    set filterout [get_config $a filterout]
    foreach filter $filterout {
	set idx [lsearch -exact $guess $filter]
	if {$idx != "-1"} {
	    set guess [KobazTools::ldelete $guess $idx]
	}
    }
    
    set guess  [string tolower $guess]
    if {![string match *$answer* $guess]} {return}

    debug "GUESS: $nick got answer of: $guess"
    KobazTools::tellnick pub $channel "Yay! $nick got the answer. The answer is: $cur($channel,af)"
    
    set_score $nick $channel 1 +1
    
    set gametype [get_config ${a}_$channel gametype]
    if {$gametype == "1"} {
        set maxrounds [get_config ${a}_$channel rounds]
        if {$cur($channel,round) >= $maxrounds} {
            KobazTools::tellnick pub $channel "Trivia Game has ended. Final scores:"
            show_scores $channel win
            return
        }
    } else {
        set maxpoints    [get_config ${a}_$channel points]
        set highestscore [get_highestscore $channel]

        if {$highestscore >= $maxpoints} {
            KobazTools::tellnick pub $channel "Trivia Game has ended. Final scores:"
            show_scores $channel win
            return
        }
    }

    KobazTools::tellnick pub $channel "Round has ended. Current scores:"
    
    kill_round $channel
    show_scores $channel
    
    incr cur($channel,round)

    timer [get_config ${a}_$channel idlekilltime] "KobazTrivia::idletimer $channel"
    trivia $channel 0

    return
}

proc show_scores {channel {win ""}} {
    set channel [string tolower $channel]

    set numrecs [KobazDB::dbNumRecs kbtrivia_scores_$channel]
    if {$numrecs == 0} {
	KobazTools::tellnick pub $channel "No one got any points :("
	return
    }

    set last [expr $numrecs - 1]
    KobazDB::dbSort kbtrivia_scores_$channel score -decreasing
    if {[KobazDB::dbField kbtrivia_scores_$channel score $last] == 0} {
	KobazTools::tellnick pub $channel "No one got any points :("
	return
    }
    
    set scorelist ""
    for {set i 0} {$i < $numrecs} {incr i} {
        set nick  [KobazDB::dbField kbtrivia_scores_$channel nick $i]
        set score [KobazDB::dbField kbtrivia_scores_$channel score $i]
        lappend scorelist "$nick $score"
    }
    
    set highestwho   [lindex [lindex $scorelist 0] 0]
    set highestscore [lindex [lindex $scorelist 0] 1]

    KobazDB::dbSort kbtrivia_scores_$channel nick

    # display scores

    set scorestuff ""
    set scoresec   "0"
    set numscores  "1"
    
    foreach nick $scorelist {
	set nickscore [lindex $nick 1]
	set nick      [lindex $nick 0]

	if {$scorestuff == ""} {
	    set scorestuff "\[${nick} - $nickscore\]"
	} else {
	    set scorestuff "$scorestuff \[${nick} - $nickscore\]"
	}

	if {[string length $scorestuff] > 60} {
            set scores($scoresec) $scorestuff
            incr scoresec
            set numscores 0
            set scorestuff ""
	}

        incr numscores
    }

    set scores($scoresec) $scorestuff
    for {set x 0} {$x <= $scoresec} {incr x} {
        KobazTools::tellnick pub $channel "$scores($x)"
    }


    set numtie "0"    
    set scorelist [lreplace $scorelist 0 0]
    set tie ""

    set numscores [llength $scorelist]
    set count 0
    foreach nick $scorelist {
	set nickscore [lindex $nick 1]
	set nick      [lindex $nick 0]

	if {$tie == ""} {set com ""} else {set com ","}

	if {$nickscore == $highestscore} {
	    set tie "$tie$com ${nick}"
	    incr numtie
	}
	incr count
    }

    if {$win == ""} {
	set message "has the highest score!"
    } else {
	set message WINS!
    }

    if {$tie == ""} {
	KobazTools::tellnick pub $channel "$highestwho $message"
    } else {
	if {$numtie == 1} {
	    KobazTools::tellnick pub $channel "${highestwho} and${tie} are\
		    tied!"
	    return
	}
	if {$numtie > "1"} {
	    set lasttie [llength $tie]
	    set tie [linsert $tie [expr $lasttie -1] "and"]
	}
	KobazTools::tellnick pub $channel "${highestwho}, ${tie} are tied!"
    }

    if {$win == "win"} {
	kill_game $channel
    }

    return
}

proc trivia {channel timesup} {
    set channel [string tolower $channel]

    variable cur
    variable a

    debug "TRIVIA: Round   : $cur($channel,round)"

    set gametype [get_config ${a}_KobazTrivia gametype]

    if {$timesup == "2"} {
	KobazTools::tellnick pub $channel "No one got the answer yet. \
             Hurry up you have [expr [get_config ${a}_$channel roundtime] / 2] \
             seconds left to answer!"

	KobazTools::tellnick pub $channel "Q: $cur($channel,qf)"
	set halftimer [expr [get_config ${a}_$channel roundtime] / 2]
	utimer $halftimer "KobazTrivia::trivia $channel 1"
	return
    }
    if {$timesup == "1"} {
	KobazTools::tellnick pub $channel "TIMES UP!"
        set showanswer [get_config ${a}_$channel showanswer]
        if {$showanswer} {
            KobazTools::tellnick pub $channel "The answer was: $cur($channel,af) "
        }
	if {$gametype == "1"} {
	    set maxrounds [get_config ${a}_$channel rounds]
	    if {$cur($channel,round) >= $maxrounds} {
		KobazTools::tellnick pub $channel "Trivia Game has ended. Final scores:"
		show_scores $channel win
		return
	    }
	} else {
	    set maxpoints [get_config ${a}_$channel points]
            set highestscore [get_highestscore $channel]
	    if {$highestscore == $maxpoints} {
		KobazTools::tellnick pub $channel "Trivia Game has ended. Final scores:"
		show_scores $channel win
		return
	    }
	}

	incr cur($channel,round)
	KobazTools::tellnick pub $channel "Round has ended. Current scores:"

        kill_round  $channel
	show_scores $channel
	trivia      $channel 0
	return
    }

    if {$gametype == "1"} {
	KobazTools::tellnick pub $channel "Starting round: $cur($channel,round) of [get_config ${a}_$channel rounds]."
    } else {
	KobazTools::tellnick pub $channel "Starting round: $cur($channel,round)"
    }

    set filterout [get_config $a filterout]
    set datafile  [get_config ${a}_$channel datafile]

    set cur($channel,inround) "1"
    set cur($channel,q)       [get_question $channel]
    set cur($channel,subject) [dbField $datafile subject  $cur($channel,q) absolute]
    set cur($channel,qf)      [dbField $datafile question $cur($channel,q) absolute]
    set cur($channel,af)      [dbField $datafile answer   $cur($channel,q) absolute]
    set cur($channel,af)      [string tolower $cur($channel,af)]

    foreach filter $filterout {
	set idx [lsearch -exact $cur($channel,af) $filter]
	if {$idx != "-1"} {
	    set cur($channel,af) [KobazTools::ldelete $cur($channel,af) $idx]
	}
    }

    debug "TRIVIA: Subject : $cur($channel,subject)"
    debug "TRIVIA: Question: $cur($channel,qf)"
    debug "TRIVIA: Answer  : $cur($channel,af)"

    KobazTools::tellnick pub $channel "Subject: $cur($channel,subject)"
    KobazTools::tellnick pub $channel "Q: $cur($channel,qf)"
    KobazTools::tellnick pub $channel "You have [get_config ${a}_$channel roundtime] seconds\
         to get the answer! Just type \"=\" and then what you think is the answer."

    utimer 15 "utimer [expr [get_config ${a}_$channel roundtime] / 2] \"KobazTrivia::trivia $channel 2\""
    return
}

proc pub_add {nick uhost handle channel arg} { 
    variable a
    variable add

    set channel [string tolower $channel]
    if {[catch {datafile_load_forchannel $a $channel} error]} {putlog $error; return}

    if {[get_config ${a}_$channel enable] == 0} {return}
    if {![check_access $nick $channel $handle\
	    [get_config ${a}_$channel !addtrivia]\
	    [get_config ${a}_$channel msg]]} {
	return
    }

    set who $nick
    set errortype notice

    set add($nick,perm) "1"
    set add($nick,chan) "$channel"
    KobazTools::tellnick msg $nick "Submiting Questions to $channel."
    KobazTools::tellnick msg $nick "Type !addtrivia subject ; question ; answer"
    KobazTools::tellnick msg $nick "Remember to keep answers simple, write \"5\" instead\
         of \"five\", and \"dog\" instead of \"a dog\". Please don't waste your time and\
         mine by submitting stupid questions."

    KobazTools::tellnick msg $nick "You can now keep adding questions using !addtrivia via /msg"
    return
}

proc msg_add {nick uhost handle arg} {
    variable a
    variable add
    variable add

    if {![info exists add($nick,perm)]} {return}
    set channel $add($nick,chan)
    set channel [string tolower $channel]
    if {[catch {datafile_load_forchannel $a $channel} error]} {putlog $error; return}

    if {[get_config ${a}_$channel enable] == 0} {return}
    if {![check_access $nick $channel $handle\
	    [get_config ${a}_$channel !addtrivia]\
	    [get_config ${a}_$channel msg]]} {
	return
    }

    set datafile [get_config ${a}_$channel datafile]

    if {$arg == "undo" && [info exists add($nick,last)]} {
	debug "ADDTRIV: $nick did an undo"
	KobazTools::tellnick msg $nick "Removing last submitted question of: $add($nick,last)"
	set rec [dbFind $datafile question $add($nick,last)]
	dbDelete $datafile $rec
	unset add($nick,last)
	return
    }

    if {$arg != ""} {
	set entry [split $arg ";"]
	
	set subject  [string trim [lindex $entry 0] " "]
	set question [string trim [lindex $entry 1] " "]
	set answer   [string trim [lindex $entry 2] " "]

	if {$subject == "" || $question == "" || $answer == ""} {
	    KobazTools::tellnick msg $nick "OOPS! You forgot something. !addtrivia subject ; question ; answer"
	    return
	}

	if {[info exists add($nick,last)] && $add($nick,last) == $question} {
	    KobazTools::tellnick msg $nick "You just submitted that question."
	    return
	}

	if {[dbBinarySearch $datafile question [string tolower $question]] != "-1"} {
	    KobazTools::tellnick msg $nick "That question exists."
	    return
	}
	
	KobazTools::tellnick msg $nick "Subject : $subject"
	KobazTools::tellnick msg $nick "Question: $question"
	KobazTools::tellnick msg $nick "Answer  : $answer"
	KobazTools::tellnick msg $nick "Question submitted, if there is an error please type\
		!addtrivia undo"
	set add($nick,last) $question

	debug "ADDTRIV: -=- $nick Submitted question -=-"
	debug "ADDTRIV: Subject : $subject"
	debug "ADDTRIV: Question: $question"
	debug "ADDTRIV: Answer  : $answer"

#	exec echo "Q  : $question" >> /home/kobaz/trivia/added
#	exec echo "WHO: $nick    " >> /home/kobaz/trivia/added

	set adding ""
	lappend adding $subject
	lappend adding $question
	lappend adding $answer

	dbInsert $datafile $adding
	datafile_save $a $datafile
	return
    } else {
	KobazTools::tellnick msg $nick "OOPS! You forgot something.\
		!addtrivia subject ; question ; answer"	
	return
    }
    return
}

proc get_question {channel {question ""}} {
    set channel [string tolower $channel]

    variable cur
    variable a

    set datafile [get_config ${a}_$channel datafile]
    set numtriv [dbNumRecs $datafile]

    set qnum [KobazTools::random $numtriv]

    # Check if we have already used every single question
    # If we have then find one we havent used, if 
    # we haven't then we must use one that we used before
    if {$numtriv > [llength $cur($channel,aqs)]} {
        while {[lsearch $cur($channel,aqs) $qnum] != "-1"} {
            set qnum [KobazTools::random $numtriv]
        }
    } else {
        set qnum [KobazTools::random $numtriv]
    }

    debug "GETQ: Random number out of: $numtriv is: $qnum"
    lappend cur($channel,aqs) $qnum

    return $qnum
}

init

}



